unit MICAMMainUnit;
{
MICAM uses a Microscope camera or Webcam to make microscope photographs
and do measurements on those photographs.
Copyright (C) 2006  Marien van Westen  mvanwesten@home.nl

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
}

interface

uses
  Windows, Types, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, StdCtrls, ComCtrls, Buttons, Clipbrd, FileCtrl, JPGComment,
  ExtDlgs, Variants, DateUtils, StrUtils, VCap, MMSystem, Math, SysConst;


type

  ObjCal =record
    Obj : Real;
    ObjStr : String;
    Cal : Real;
    end;

  TMicroscope = record
    Name: String;
    CaptureWidth : Integer;
    NrofObjectives: Integer;
    Objective: Array[0..31] of ObjCal;
    end;

   MP = record
     Width : Integer;
     Color: Integer;
     end;

   TMain = class(TForm)
    MainMenu: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    SelectVideoDevice: TMenuItem;
    StatusBar: TStatusBar;
    About1: TMenuItem;
    Help: TMenuItem;
    Help2: TMenuItem;
    Preferences: TMenuItem;
    OpenImage1: TMenuItem;
    Measurement: TMenuItem;
    Clear: TMenuItem;
    InsertScale1: TMenuItem;
    SaveImageAs1: TMenuItem;
    SaveUseOptions: TMenuItem;
    OpenDialog: TOpenPictureDialog;
    SaveDialog: TSavePictureDialog;
    InsertCopyright1: TMenuItem;
    Tools: TMenuItem;
    Timer: TTimer;
    SelectDevice: TMenuItem;
    EditCalibrationValue: TMenuItem;
    Search: TMenuItem;
    CutSelection1: TMenuItem;
    CutSelection2: TMenuItem;
    CutSelection3: TMenuItem;
    Windows: TPageControl;
    PreviewTab: TTabSheet;
    VidCapBox: TScrollBox;
    MeasuringTab: TTabSheet;
    FileList: TListBox;
    MeasureBox: TScrollBox;
    Foto: TPaintBox;
    CommentMemo: TMemo;
    SystemCommentLabel: TLabel;
    UserCommentLabel: TLabel;
    UserMemo: TMemo;
    SavePicBox: TGroupBox;
    SavePicture: TButton;
    OpenPicture: TButton;
    SaveOptions: TButton;
    Objective: TGroupBox;
    ObjInfo: TLabel;
    RB1: TRadioButton;
    RB2: TRadioButton;
    RB3: TRadioButton;
    RB4: TRadioButton;
    RB5: TRadioButton;
    RB6: TRadioButton;
    ObjectivesBox: TComboBox;
    FrameCapture: TGroupBox;
    Snapshot: TButton;
    SnapWithOptions: TButton;
    FitToPreviewWindow: TCheckBox;
    CalibrationBox: TGroupBox;
    CalLabel: TLabel;
    Label1: TLabel;
    Label4: TLabel;
    Calibrate: TCheckBox;
    Action: TEdit;
    Calibration: TEdit;
    FotoSize: TComboBox;
    CaptureBox: TGroupBox;
    Label3: TLabel;
    Counter: TLabel;
    ToMeasurement: TButton;
    CBNoiseReduction: TCheckBox;
    NrBox: TComboBox;
    HWTimer: TTimer;
    CutSelection4: TMenuItem;
    InsertScale2: TMenuItem;
    InsertScale3: TMenuItem;
    InsertScale4: TMenuItem;
    ExeNameLabel: TLabel;
    ExperimentalLabel: TLabel;
    InsertLabel: TMenuItem;
    Delay: TTimer;
    AVIBox: TGroupBox;
    AVIStartStop: TBitBtn;
    procedure SelectDeviceClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SnapshotClick(Sender: TObject);
    procedure ToMeasurementClick(Sender: TObject);
    procedure AVIStartStopClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure ObjectiveClick(Sender: TObject);
    procedure Help2Click(Sender: TObject);
    procedure PreferencesClick(Sender: TObject);
    procedure OpenImageClick(Sender: TObject);
    procedure FotoMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FotoMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FotoMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure FitToMeasurementWindowClick(Sender: TObject);
    procedure WindowsChange(Sender: TObject);
    procedure ClearClick(Sender: TObject);
    procedure InsertScaleClick(Sender: TObject);
    procedure PlaceScale(X,Y:Integer);
    Procedure PlaceString(Str:String;X,Y:Integer);
    procedure MoveSelection(X,Y : Integer);
    procedure CutSelection(X,Y:Integer);
    procedure CancelCutSelection;
    procedure MeasureAndCalibrate(X,Y :Integer);
    procedure DrawLine(X,Y : Integer);
    procedure FotoPaint(Sender: TObject);
    procedure SaveImageAsClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FileSearch(const PathName, FileName : string);
    procedure Next;
    procedure Previous;
    procedure DoubleSize(Src:TBitmap;var Dst:TBitmap);
    procedure StackIt(var Dst:TBitmap);
    procedure CBNoiseReductionClick(Sender: TObject);
    procedure SaveUseOptionsClick(Sender: TObject);
    procedure SnapWithOptionsClick(Sender: TObject);
    procedure InsertCopyright1Click(Sender: TObject);
    procedure FillComment;
    procedure TimerTimer(Sender: TObject);
    Procedure Display(Filename:String);
    procedure VSClick(Sender: TObject);
    procedure capBitmapGrabbed(CapturedImage: TCapturedBitmap);
    procedure GetCapturedFrame(var bmp: TBitmap);
    procedure CaptureFrame;
    procedure SetVidCapSize;
    procedure capFrameRendered(Sender: TObject);
    procedure capStartPreview(Sender: TObject);
    procedure capStopPreview(Sender: TObject);
    procedure capAborted(Sender: TObject);
    procedure capDeviceLost(Sender: TObject);
    procedure capChangeDevice(Sender: TObject);
    procedure capCaptureProgress(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure EditCalibrationValueClick(Sender: TObject);
    procedure ObjectivesBoxClick(Sender: TObject);
    procedure SearchClick(Sender: TObject);
    procedure CutSelectionClick(Sender: TObject);
    procedure StartHWTimer;
    procedure HWTimerTimer(Sender: TObject);
    procedure InsertLabelClick(Sender: TObject);
    procedure DelayTimer(Sender: TObject);
    procedure SetSnapshotSize(var bmp2:TBitmap; bmp1:TBitmap);
    procedure WindowsMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
   originalPanelWindowProc : TWndMethod;
   procedure PanelWindowProc (var Msg : TMessage) ;
   procedure PanelImageDrop (var Msg : TWMDROPFILES) ;
    { Private-Declarations}
  public
    VersionStr :String;
    VidCap: TVideoCapture;
    config: TGraphConfig;
    VideoDevice : String;
    VideoWidth : Integer;
    VideoHeight : Integer;
    Compressor : String;
    ProgDir :String;
    FrameDir:String;
    AVIDir  :String;
    Orgbmp, Tekenbmp : TBitmap;
    MeasurePen : MP;
    ScalePen : MP;
    ScaleLength : Integer;
    CalValue : Real;
    ImgCalValue : Real;
    Microscope : Array [-1..15] of TMicroscope; // -1 used for microscope from file
    DefaultMic : Integer;
    DefaultObj: Integer;
    SelectedMic: Integer;
    SelectedObj: Integer;
    ImageObj:Real;
    NrofMicroscopes : Integer;
    RB : Array[0..5] of TRadioButton;
    SnapshotWarning : Boolean;
    SnapSize : Integer;
    ImageKind: Integer;       //0=BMP 1=JPG  2=PNG
    CaptureFileType: Integer; // 0=Date+Nr+Magn  1=Date+Nr  2=Name+Number+Magn
    CaptureNr: Integer;       // nr used in filename
    CaptureName: String;
    CaptureDoubleSized: Boolean;
    JpgComment: TJpgComment;
    CommentList: TStringlist;   // list containing comment for image
    SoundEnabled : Boolean;
    Copyrightstring: String;
    LabelString: String;
    CopyRightFont: TFont;
    Units: String;
    SampleDate: TDate;
    LocationText1 : String;
    LocationText2 : String;
    BMPcomment: Boolean;
    UseHWTimer : Boolean;
    PoortNr : Integer;
    PoortLine: Integer;   // 0 CTS  1= DSR  2 = DCD,  3 = RI
    PoortActive: Boolean;
    PoortHigh : Integer;  // 0..3  0 none high, 1 RTS high, 2 DTR high, 3 both high
    OldTrig, NewTrig : Boolean;
    UseDeviceDefaultSettings : Boolean;
    NoDevice: Boolean;
    WarningOnResize: Boolean;
    WarningOnAVI : Boolean;
    Procedure DoCalibrate;
    Procedure DoMeasure;
    procedure SetObjectives(CurrentMic, CurrentObj:Integer);
    procedure SetObjectivesDropDown(CurrentMic, CurrentObj:Integer);
    procedure SetObjectivesRadioButton(CurrentMic, CurrentObj:Integer);
    procedure OpenImgFile(Filename:String);
    function CreateFilename(Kind: Integer): String;
    Procedure StoreComment(UsrStr,SystemStr: TStrings);
    procedure GetInfoFromComment;
    function ReadUserComment:String;
    function ReadSystemComment: String;
    procedure ReadMicroscopeFromComment;
    function ExtStr : String;
    procedure CheckScrollBar(Memo: TMemo);
    procedure MakeVidcap;
    procedure SetCalibrationText(CalValue: Real);

    { Public-Declarations}
  end;

var
  Main: TMain;
  X1, X2, Y1, Y2 : Integer;
  sw, sh, fsw, fsh : Integer;                 // width and height of selected area
  TheRect: Trect;
  ImgSourceIsFile: Boolean;
  Teller : Integer;
  MenuAction: Integer;             // if MenuAction=0 do nothing
                                   // if MenuAction=1 then PlaceScale(X,Y);
                                   // if MenuAction=2 then PlaceCopyright(X,Y);
                                   // if MenuAction=3 then SelectCut(X,Y);
                                   // if MenuAction=4 then PlaceLabel(X,Y);
  MemImage : array[0..3200,0..2400, 1..3] of Integer;  // image operations
  Magnification: Real;                 // Magnification of the objective used
  BitmapReady : Boolean;               // To check if the captured frame is ready
  lbdialogs : TStringList;             // store VCap dialog objects
  Hiddenbmp : TBitmap;                 // Temp bmp when capturing
  MoveSelect : Boolean;
  Muis : Integer;

const
  VERSION =1;
  MAJOR = 2;
  MINOR = 2;
  CREATIONDATE = 'January 2009';

  Down =1;
  Up=0;

implementation

{$R *.DFM}

uses Consts, DlgDriver, Preferences, AboutMICAM, SaveOptions, HelpUnit,
  MicamUtils, ShellAPI, ActiveX, SearchUnit, Serial, VCapStrings;


///////////////////////////////////////////////////////////////////
//
//        Create and Destroy
//
///////////////////////////////////////////////////////////////////


procedure TMain.FormCreate(Sender: TObject);
var
   i,j :Integer;
   StringList : TStringList;
   Flag : Boolean;
begin
  VersionStr := 'MICAM version '+IntToStr(VERSION)+'.'+IntToStr(MAJOR)+'.'+IntToStr(MINOR);
  ExperimentalLabel.Caption:='Experimental '+IntToStr(VERSION)+'.'+IntToStr(MAJOR)+'.'+IntToStr(MINOR);
  if MINOR=0 then
     VersionStr := 'MICAM version '+IntToStr(VERSION)+'.'+IntToStr(MAJOR);
  ExeNameLabel.Caption:=ChangeFileExt(ExtractFileName(Application.ExeName),'');

  MenuAction:=0;
  SampleDate:=Today;
  CopyRightString:='Made with '+Versionstr;
  CopyRightFont:=TFont.Create;
  CopyrightFont.Charset:=DEFAULT_CHARSET;
  originalPanelWindowProc := MeasureBox.WindowProc;    // for drag and drop
  MeasureBox.WindowProc := PanelWindowProc;            // for drag and drop
  DragAcceptFiles(MeasureBox.Handle,true) ;

 for i:= -1 to 15 do
   begin
   Microscope[i].Name:='NoName';
   for j:=0 to 5 do
   begin
    Microscope[i].Objective[j].Obj:=0;
    Microscope[i].Objective[j].Cal:=0;
   end;
   end;
 Orgbmp:=TBitmap.Create;
 Tekenbmp:=TBitmap.Create;
 MeasuringTab.DoubleBuffered:=True;
 ImgSourceIsFile:=False;
 jpgComment:=TjpgComment.Create(Self);
 CommentList:=TStringlist.Create;
 lbdialogs:=TStringlist.Create;
 Hiddenbmp:=Tbitmap.Create;
 LocationText1 :='';
 LocationText2 :='';
 CaptureDoubleSized:=False;
 SnapSize:=0;
 ImageKind:=1;
 BMPcomment:=True;
 Progdir:=ExtractFilePath(Application.ExeName);
 if not DirectoryExists(ProgDir+'\Capture') then CreateDir(ProgDir+'\Capture');

 //Progdir:=ExtractFileDir(paramstr(0));

 MakeVidcap;
 config:= TGraphConfig.Create;
 RB[0]:=RB1;
 RB[1]:=RB2;
 RB[2]:=RB3;
 RB[3]:=RB4;
 RB[4]:=RB5;
 RB[5]:=RB6;

 LoadPrefsFromIni;

 If NoDevice=False then
 begin
 Flag:=False;
 StringList := GetVideoDevicesList;
 i:=Stringlist.Count;
   try
     for i:= 0 to Stringlist.Count -1 do
       if VideoDevice=Stringlist[i] then flag:=True
   finally
     Stringlist.Free;
   end;
   if (Flag=False) and (VideoDevice<>'') then
     begin
     MessageDlg('Unable to open last used device',mtError, mbOKCancel,0);
     VideoDevice:='';
     end;

 if (VideoDevice<>'') and (UseDeviceDefaultSettings) then
    begin
    VidCap.SaveGraph(config);
    config.VCapSource:= VideoDevice;
    config.VComp:=Compressor;
    config.Vcapmode.Width:=VideoWidth;
    config.VCapMode.Height:=VideoHeight;
    config.VCapMode.FrameRate:=10;
    Vidcap.FrameRate:=10;
    VidCap.UseFrameRate:=True;
    VidCap.RestoreGraph(config);
    Delay.Enabled:=True;   // for some reason you must wait some time before changing resolution. Who knows why?
   end;
   end; // if NoDevice=False

   if NoDevice=True then
    begin
    Windows.ActivePage:=MeasuringTab;
    WindowsChange(nil);
    end;

 SaveDialog.InitialDir:=FrameDir;
 OpenDialog.InitialDir:=FrameDir;
 SetObjectives(SelectedMic, SelectedObj);
 Microscope[-1]:=Microscope[SelectedMic];

 if Paramstr(1)<> '' then
    begin
    OpenDialog.InitialDir:=ExtractFileDir(paramstr(1));
    OpenImgFile(Paramstr(1));
    FileList.Clear;
    FileSearch(ExtractFilePath(Paramstr(1)), '*.*');
    i:=-1;
    repeat
     inc(i);
    until (Paramstr(1)=FileList.Items[i]) or (i>FileList.Count);
    Teller:=i;
    end;
end;

procedure TMain.FormDestroy(Sender: TObject);
begin
 if jpgComment<> nil then jpgComment.Free;
 if CommentList<> nil then CommentList.Free;
 if lbdialogs<> nil then lbdialogs.Free;
 if Hiddenbmp<> nil then Hiddenbmp.Free;
end;


procedure TMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  CloseCom;
  SavePrefsToIni;
  VidCap.StopPreview;
  VidCap.StopCapture;
end;

procedure TMain.SetObjectives(CurrentMic, CurrentObj:Integer);
var
 i : Integer;
begin
 for i:=0 to 5 do
    RB[i].Visible:=False;
 ObjectivesBox.Visible:=False;
 ObjInfo.Visible:=False;
 if Microscope[CurrentMic].NrofObjectives>6
    then SetObjectivesDropDown(CurrentMic,CurrentObj)
    else SetObjectivesRadioButton(CurrentMic,CurrentObj);
 if ImgSourceIsFile=False then FillComment;
end;

procedure TMain.SetObjectivesDropDown(CurrentMic,CurrentObj:Integer);
var
 I : Integer;
begin
  ObjectivesBox.Visible:=True;
  Objective.Caption:= 'Micr.: '+Microscope[CurrentMic].Name;
  ObjectivesBox.Items.Clear;
  for i:=0 to Microscope[CurrentMic].NrofObjectives-1 do
    ObjectivesBox.Items.Add(FloatToStr(Microscope[CurrentMic].Objective[i].Obj)+ ' X');

  ObjInfo.Caption:='Calibrated';
  if Microscope[CurrentMic].Objective[CurrentObj].Cal=0 then ObjInfo.Caption:='Not Calibrated';
  if Microscope[CurrentMic].Objective[CurrentObj].Cal<0 then ObjInfo.Caption:='Calculated Calibration';
  ObjInfo.Visible:=True;
  ObjectivesBox.ItemIndex:=CurrentObj;
  CalValue := Abs(Microscope[CurrentMic].Objective[CurrentObj].Cal);
  If ImgsourceIsFile=False
        then
           begin
           Calvalue:=Calvalue * Microscope[CurrentMic].CaptureWidth/VideoWidth;
           if CaptureDoubleSized=true then CalValue:=Calvalue/2;
           end
        else
           begin
           CalValue:=ImgCalValue*ImageObj/Microscope[CurrentMic].Objective[CurrentObj].Obj;
           if OrgBmp.Width<>0 then CalValue:=CalValue* Foto.Width / OrgBmp.Width;
           end;
  if Windows.ActivePage=MeasuringTab then CalValue:=Calvalue*OrgBmp.Width/Foto.Width;
  SetCalibrationText(CalValue);
end;

procedure TMain.ObjectivesBoxClick(Sender: TObject);
begin
  SelectedObj:=ObjectivesBox.ItemIndex;
  if Objective.Font.Color=clRed
     then
       begin
       SetObjectivesDropDown(-1, SelectedObj);
       CommentMemo.Lines[3]:=(Microscope[-1].Name+';  Obj: '+ FloatToStr(Microscope[-1].Objective[SelectedObj].Obj)
               + ' X;  um/pixel: '+FloatToStrF(CalValue,ffFixed,4,6));
       end
     else
       begin
       SetObjectivesDropDown(SelectedMic, SelectedObj);
       FillComment;
       end;
end;

procedure TMain.SetObjectivesRadioButton(CurrentMic, CurrentObj:Integer);
var
  i : Integer;
begin
  for i:=0 to Microscope[CurrentMic].NrofObjectives-1 do
     RB[i].Caption:='---';
  if NrofMicroscopes=0 then Exit;
  Objective.Caption:= 'Micr.: '+Microscope[CurrentMic].Name;
  for i:=0 to Microscope[CurrentMic].NrofObjectives-1 do
    begin
     RB[i].Visible:=True;
     if i=CurrentObj then RB[i].Checked:=True else RB[i].Checked:=False;
     RB[i].Caption:= FloatToStr(Microscope[CurrentMic].Objective[i].Obj)+' X';
     if Microscope[CurrentMic].Objective[i].Obj=0 then RB[i].Visible:=False;
     if Microscope[CurrentMic].Objective[i].Cal=0
        then RB[i].Font.Color:=clRed
        else if Microscope[CurrentMic].Objective[i].Cal<0
                then RB[i].Font.Color:=clGreen
                else RB[i].Font.Color:=clBlack;
    end;
  CalValue := Abs(Microscope[CurrentMic].Objective[CurrentObj].Cal);
  If ImgsourceIsFile=False
        then
           begin
           Calvalue:=Calvalue * Microscope[CurrentMic].CaptureWidth/VideoWidth;
           if CaptureDoubleSized=true then CalValue:=Calvalue/2;
           end
        else
           begin
           CalValue:=ImgCalValue*ImageObj/Microscope[CurrentMic].Objective[CurrentObj].Obj;
           if OrgBmp.Width<>0 then CalValue:=CalValue* Foto.Width / OrgBmp.Width;
           end;
  if Windows.ActivePage=MeasuringTab then CalValue:=Calvalue*OrgBmp.Width/Foto.Width;
  SetCalibrationText(CalValue);
end;

procedure TMain.ObjectiveClick(Sender: TObject);
begin
  SelectedObj:=TRadioButton(Sender).Tag;
  if Objective.Font.Color=clRed
     then
       begin
       SetObjectivesRadioButton(-1, SelectedObj);
       CommentMemo.Lines[3]:=(Microscope[-1].Name+';  Obj: '+ FloatToStr(Microscope[-1].Objective[SelectedObj].Obj)
               + ' X;  um/pixel: '+FloatToStrF(CalValue,ffFixed,4,6));
       end
     else
       begin
       SetObjectivesRadioButton(SelectedMic, SelectedObj);
       FillComment;
       end;
end;


///////////////////////////////////////////////////////////////////
//
//        VideoCap Settings
//
///////////////////////////////////////////////////////////////////

procedure Tmain.MakeVidcap;
begin
  VidCap := TVideoCapture.Create(VidCapBox);
  VidCap.Parent := TWincontrol(VidCapBox);
 with VidCap do
  begin
      Left := 1;
      Top := 1;
      Width := 640;
      Height := 480;
      DVResolution := dvrDontWorry;
      WantBitmaps := True;
      BitmapPixelFormat := pf24bit;
      WantAudio := False;
      WantDVAudio := False;
      WantAudioPreview := False;
      WantPreview := True;
      WantCapture := True;
      UseFrameRate := True;
      FrameRate := 20 ;
      CaptureFileName := ProgDir+'\Capture\'+ 'capture.avi';
      UseTempFile := True ;
      TempCaptureFileName :=ProgDir+'\Capture\'+ 'captemp.avi';
      UseTimeLimit := True ;
      TimeLimit := 30;
      DoPreallocFile := False;
      PreallocFileSize := 1;
      OnAborted := capAborted;
      OnDeviceLost := capDeviceLost;
      OnChangeDevice := capChangeDevice ;
      OnCaptureProgress := capCaptureProgress ;
      OnStopPreview := capStopPreview;
      OnStartPreview := capStartPreview ;
      OnBitmapGrabbed := capBitmapGrabbed ;
      OnFrameRendered := capFrameRendered;
      //Align := alClient;
      Color := clBlue;
    end;
end;


procedure TMain.SelectDeviceClick(Sender: TObject);
begin
  if DriverDlg.Showmodal=mrCancel then Exit;
  if DriverDlg.DeviceBox.ItemIndex < 0 then Exit;
  VidCap.StopPreview;
  VidCap.StopCapture;
  Vidcap.OnChangeDevice(nil);
  Vidcap.Free;
  MakeVidcap;

  VidCap.SaveGraph(config);
  config.ACapSource:='';
  config.VCapSource:= DriverDlg.DeviceBox.Items[DriverDlg.DeviceBox.ItemIndex];
  config.VComp:=DriverDlg.CompressorBox.Items[DriverDlg.CompressorBox.ItemIndex];
  VidCap.RestoreGraph(config);
  VideoDevice:=config.VCapSource;
  LoadCameraInfo(VideoDevice);
  Delay.Enabled:=True;
  Compressor:=config.VComp;
  if not VidCap.Init then begin ShowMessage('Can''t init capture!'); Exit; end;

  If  NrofMicroscopes=0 then
     MessageDlg('You have to create at least ONE microscope in preferences'+#10#13+
                 'and calibrate it first for proper operation of the program',mtWarning,[mbOK],0);
end;

procedure TMain.CaptureFrame;
begin
  BitmapReady:=False;
  VidCap.CaptureFrame;
end;

procedure TMain.GetCapturedFrame(var bmp: TBitmap);
begin
 repeat
 Application.ProcessMessages;
 until BitmapReady=True;
 bmp.Assign(Hiddenbmp);
end;

procedure TMain.capBitmapGrabbed(CapturedImage: TCapturedBitmap);
begin
  Hiddenbmp.Assign(CapturedImage);
  BitmapReady:=True;
end;

procedure TMain.capFrameRendered(Sender: TObject);
begin
  //framecount:= framecount+1;
 // StatusBar.Panels[0].Text:= Format('Frames passed: %d', [framecount]);
end;

procedure TMain.capStartPreview(Sender: TObject);
begin
  FrameCapture.Enabled:=True;
end;

procedure TMain.capStopPreview(Sender: TObject);
begin
 FrameCapture.Enabled:=False;
 // framecount:= 0;
 //StatusBar.Panels[0].Text:='';
end;

procedure TMain.capAborted(Sender: TObject);
begin
// normal graph run has been aborted
end;

procedure TMain.capDeviceLost(Sender: TObject);
begin
// capture device has been lost
end;

procedure TMain.capChangeDevice(Sender: TObject);
var
  NewItem : TMenuItem;
  d: TCaptureDialog;
  Str : String;
begin
    MainMenu.Items[1].Clear;
    NewItem := TMenuItem.Create(Self);
    NewItem.Caption := 'Select Device';
    NewItem.OnClick:=SelectDeviceClick;
    MainMenu.Items[1].Add(NewItem);
    if Vidcap.Previewing=False then
      begin
      Exit;
      end;
    lbDialogs.Clear;
    for d:= Low(TCaptureDialog) to High(TCaptureDialog) do
    if d in VidCap.Dialogs then begin
      lbDialogs.AddObject(DialogTitles[d], TObject(d));
         NewItem := TMenuItem.Create(Self);
         Str:= DialogTitles[d];
         if Str ='Video Capture Filter' then Str :='Video Source Settings';
         if Str ='Video Capture Pin' then Str :='Video Format Settings';
         NewItem.Caption := Str;
         NewItem.OnClick:=VSClick;
         MainMenu.Items[1].Add(NewItem);
        end;
    SetVidCapSize;
end;

procedure TMain.SetVidCapSize;
var
  w, h : Integer;
  f : Real;
begin

  if FitToPreviewWindow.Checked=False then
     VidCap.SetBounds(0,0, VidCap.VCapMode.Width, VidCap.VCapMode.Height);
  if FitToPreviewWindow.Checked=True then
     begin
     w:=VidCap.VCapMode.Width;
     h:=VidCap.VCapMode.Height;
     if w<>0 then f:=h/w else f:=0.75;
     w:= VidCapBox.Parent.Width-4;
     h:= Round(w *f) ;
     if h>VidCapBox.Parent.Height-4 then
         begin
         h:= VidCapBox.Parent.Height-4;
         w:= Round(h/f);
         end;
     VidCap.SetBounds(0, 0, w, h);
     end;
end;

procedure TMain.capCaptureProgress(Sender: TObject);
begin
  StatusBar.Panels[0].Text:= 'Frames: Captured '+IntToStr(VidCap.NotDropped)+
                         '. Dropped '+IntToStr(VidCap.FramesDropped)+
                         '. Capture time '+IntToStr(VidCap.CapTime div 1000)+'.'+IntToStr((VidCap.CapTime div 10) mod 100);
  if not VidCap.Capturing then
     begin
     AVIStartStop.caption:= 'Start';
     AVIStartStop.Font.Color:= clred;
     end;                       
end;

procedure TMain.VSClick(Sender: TObject);
var
  i: Integer;
begin
  for i:=1 to MainMenu.Items[1].Count-1 do
      if Sender=MainMenu.Items[1].Items[i] then
         begin
         if MainMenu.Items[1].Items[i].Caption='Video Format' then
            VidCap.StopPreview;
         VidCap.ShowDialog(TCaptureDialog(lbDialogs.Objects[i-1]));
         VidCap.StartPreview;
         end;
   SetVidCapSize;
   VideoWidth:=VidCap.VCapMode.Width;
   VideoHeight:=VidCap.VCapMode.Height;
end;

///////////////////////////////////////////////////////////////////
//
//        End of VideoCap Settings
//
///////////////////////////////////////////////////////////////////

procedure TMain.FormShow(Sender: TObject);
begin
  If Windows.ActivePage=MeasuringTab then Exit;
  WindowsChange(nil);
end;

procedure TMain.WindowsChange(Sender: TObject);
begin
 SetObjectives(SelectedMic,SelectedObj);

 If Windows.ActivePage=PreviewTab then
    begin
    SetObjectives(DefaultMic,DefaultObj);
    Objective.Font.Color:=clDefault;
    CalibrationBox.Visible:=False;
    CommentMemo.Visible:=False;
    SystemCommentLabel.Visible:=False;
    UserMemo.Visible:=False;
    UserCommentLabel.Visible:=False;
    Capturebox.Visible:=True;
    AVIBox.Visible:=True;
    FrameCapture.Visible:=True;
    FrameCapture.BringToFront;
    SavePicBox.Visible:=False;
    Measurement.Enabled:=False;
    Tools.Enabled:=False;
    SelectVideoDevice.Enabled:=True;
    SaveImageAs1.Visible:=False;
    SaveUseOptions.Visible:=False;
    end;
 If Windows.ActivePage=MeasuringTab then
    begin
    CalibrationBox.Visible:=True;
    CommentMemo.Visible:=True;
    SystemCommentLabel.Visible:=True;
    UserMemo.Visible:=True;
    UserCommentLabel.Visible:=True;
    UserMemo.Clear;
    Capturebox.Visible:=False;
    AVIBox.Visible:=False;
    FrameCapture.Visible:=False;
    FrameCapture.SendToBack;
    SavePicBox.Visible:=True;
    Measurement.Enabled:=True;
    Tools.Enabled:=True;
    SelectVideoDevice.Enabled:=False;
    SaveImageAs1.Visible:=True;
    SaveUseOptions.Visible:=True;
    end;
end;


///////////////////////////////////////////////////////////////////
//
//        Toolpanel Preview
//
///////////////////////////////////////////////////////////////////

procedure TMain.SetSnapshotSize(var bmp2:TBitmap; bmp1:TBitmap);
begin
  Case SnapSize of
    0: begin bmp2.Width:=Bmp1.Width; bmp2.Height:=bmp1.Height; end;
    1: begin bmp2.Width:=640; bmp2.Height:=round(bmp1.Height*640/bmp1.Width); end;
    2: begin bmp2.Width:=800; bmp2.Height:=round(bmp1.Height*800/bmp1.Width); end;
    3: begin bmp2.Width:=1024; bmp2.Height:=round(bmp1.Height*1024/bmp1.Width); end;
    4: begin bmp2.Width:=1280; bmp2.Height:=round(bmp1.Height*1280/bmp1.Width); end;
    else begin bmp2.Width:=Bmp1.Width; bmp2.Height:=bmp1.Height; end;
  end;
end;

procedure TMain.SnapshotClick(Sender: TObject);
var
  Str : String;
  bmp1, bmp2:TBitmap;
  TmpRect : Trect;
begin
  if SnapShot.Enabled=False then Exit;
  CaptureFrame;;
  if SnapshotWarning=True then
    if SelectedObj<>DefaultObj then
      if MessageDlg('Objective is non-default. Continue?',mtConfirmation,[mbYes,mbNo],0)=mrNo
         then Exit;
  if SoundEnabled then PlaySound(PChar('Camera_snap.wav'), 0, SND_SYNC);
  Str:=CreateFilename(0);
  bmp1:=Tbitmap.Create;
  bmp1.PixelFormat:=pf24bit;
  bmp2:=Tbitmap.Create;
  bmp2.PixelFormat:=pf24bit;
 try
  GetCapturedFrame(bmp1);
  SetSnapshotSize(bmp2,bmp1);
  CalValue := abs(Microscope[SelectedMic].Objective[SelectedObj].Cal);
  CalValue := CalValue * Microscope[SelectedMic].CaptureWidth / bmp2.Width;
  FillComment;
  CommentMemo.Visible:=True;
  SystemCommentLabel.Visible:=True;
  Timer.Enabled:=True;
  TmpRect:=Rect(0,0,Bmp2.Width,Bmp2.Height);
  bmp2.Canvas.StretchDraw(TmpRect,bmp1);
  if not FileExists2(Str) then
         begin
         SaveBmpToFile(bmp2,Str);
         Display(Str);
         end
         else
         if MessageDlg('The file '+Str+' exists.'+#13#10+'Do you want to overwrite this file?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
            begin
            SaveBmpToFile(bmp2,Str);
            Display(Str);
            end
 finally
  bmp1.Free;
  bmp2.Free;
 end;
end;

procedure TMain.SnapWithOptionsClick(Sender: TObject);
var
  Str : String;
  bmp1, bmp2:TBitmap;
  TmpRect: TRect;
begin
  if SnapWithOptions.Enabled=False Then Exit;
  CaptureFrame;
  bmp1:=Tbitmap.Create;
  bmp1.PixelFormat:=pf24bit;
  bmp2:=Tbitmap.Create;
  bmp2.PixelFormat:=pf24bit;
  try
    GetCapturedFrame(bmp1);
    SetSnapshotSize(bmp2,bmp1);
  CalValue := abs(Microscope[SelectedMic].Objective[SelectedObj].Cal);
  CalValue := CalValue * Microscope[SelectedMic].CaptureWidth / bmp2.Width;
  FillComment;

  ImgSourceIsFile:=False;
  SaveWithOptions.Snap:=True;
  if SaveWithOptions.Showmodal = mrCancel
       then Exit
       else Str:=SaveWithOptions.Directory.Text+'\'+SaveWithOptions.SugestedFileName.Text;
   Str:=Str+'.'+SaveWithOptions.ImageType.Text;
   if FileExists2(Str) then
      if MessageDlg('The file exists. Do you want to overwrite this file?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then Exit;

   LocationText1:=SaveWithoptions.LocationEdit1.Text ;
   LocationText2:=SaveWithoptions.LocationEdit2.Text ;
   StoreComment(SaveWithOptions.UserMemo.Lines,SaveWithOptions.CommentMemo.Lines);
   Display(Str);
   Caption:=str;
   TmpRect:=Rect(0,0,Bmp2.Width,Bmp2.Height);
   bmp2.Canvas.StretchDraw(TmpRect,bmp1);
   SaveBmpToFile(bmp2,Str);
   finally
     bmp1.Free;
     bmp2.Free;
   end;
end;

procedure TMain.ToMeasurementClick(Sender: TObject);
var
 BmpTmp: TBitmap;
 Str : String;
begin
 BmpTmp:=TBitmap.Create;
 BmpTmp.PixelFormat:=pf24bit;
 BmpTmp.Width:=VideoWidth;
 BmpTmp.Height:=VideoHeight;
 if CBNoiseReduction.Checked then
    begin
      StackIt(BMPTmp);
      Orgbmp.Assign(BmpTmp);
    end
    else
    begin
      CaptureFrame;
      GetCapturedFrame(OrgBmp);
    end;
 OrgBMP.Width:=VideoWidth;
 Orgbmp.Height:=VideoHeight;
 if CaptureDoubleSized=True then
    begin
    BmpTmp.Assign(OrgBmp);
    DoubleSize(BmpTmp,Orgbmp);
    OrgBmp.Width:=VideoWidth*2;
    OrgBmp.Height:=VideoHeight*2;
    end;
 TekenBmp.Canvas.CopyRect(TheRect,Orgbmp.Canvas,TheRect);
 with Tekenbmp.Canvas do
 begin
    Pen.Width:=MeasurePen.Width;
    Pen.Color:=MeasurePen.Color;
    Font.Size:=12;
    Font.Color:=clBlack;
    Brush.Color:=clWhite;
    end;
 BmpTmp.Free;
 Str:=CreateFilename(0);
 SaveDialog.FileName:=Str;
 Display(Str);

 Windows.ActivePage:=MeasuringTab;
 WindowsChange(nil);
 FormResize(nil);
 FillComment;
end;

procedure TMain.AVIStartStopClick(Sender: TObject);
begin
  if (AVIStartStop.caption= 'Start') and VidCap.Capturing then Exit;
  if (AVIStartStop.caption= 'Start')then
     begin
     if WarningOnAVI = True
       then WarningOnAVI:=not ShowWarning(
       'Capturing an AVI file does not work on every camera!');
     config.CaptureFileName:=AVIDir+'\'+
             Inputbox('AVI filename',
             'Edit filename and press OK. Press Cancel to use this filename',
             ExtractFileName(CreateFilename(1)));
     config.UseTempFile:=True;
     config.TempCaptureFileName:=Progdir+'\Capture\captemp.avi';
     VidCap.CaptureFileName := config.CaptureFileName;
     Display(VidCap.CaptureFileName);
      // start capturing
     if VidCap.StartCapture( false ) then begin
       // vidCap.StopPreview;
       // VidCap.StartPreview;
        CapCaptureProgress (Self);
        AVIStartStop.caption:= 'Stop';
        AVIStartStop.Font.Color:= clgreen;
        end;
     end
  else
     begin
     VidCap.StopCapture;
     CapCaptureProgress (Self);
     AVIStartStop.caption:= 'Start';
     AVIStartStop.Font.Color:= clred;
     end;
end;

function TMain.CreateFilename(Kind: Integer): String;
var
  Str : String;
  Testname, Test : String;
  DateNumber: Integer;
begin
   if Kind=0 then Str:=Framedir else Str:=AVIDir;
  if CaptureFileType <= 1 then
    begin
    DateSeparator := '-';
    ShortDateFormat := 'yyyy/mm/dd';
    Str:=Str+'\'+DateToStr(SampleDate);
    DateNumber:=0;
    repeat
      TestName:=Str;
      inc(DateNumber);
      TestName:=TestName+'_'+FormatFloat('0000',DateNumber);
      Test:=TestName+'*';
      if CaptureFileType = 0 then
         TestName:=TestName+'_'+ FloatToStr(Microscope[SelectedMic].Objective[SelectedObj].Obj)+'X';
      if Kind =0 then begin TestName:=TestName+ExtStr; Test:=Test+ExtStr; end;
      if Kind =1 then begin TestName:=TestName+'.avi'; Test:=Test+'.avi';end;
    until (not FileExists2(Test)) or (DateNumber>9999);
    Result:=TestName;
    end;
  if CaptureFileType = 2 then
      begin
      Inc(CaptureNr);
      Str:=Str+'\'+CaptureName+'-'+FormatFloat('0000',CaptureNr)+'_'+ FloatToStr(Microscope[SelectedMic].Objective[SelectedObj].Obj)+'X';
      if Kind =0 then Str:=Str+ExtSTr;
      if Kind =1 then Str:=Str+'.avi';
      Result:=Str;
      end;
end;

function TMain.ExtStr : String;
begin
  Case ImageKind of
    0:  Result:='.bmp';
    1:  Result:='.jpg';
    2:  Result:='.png';
  else Result:='';
  end;
end;


///////////////////////////////////////////////////////////////////
//
//        Toolpanel Measuring
//
///////////////////////////////////////////////////////////////////

procedure TMain.OpenImageClick(Sender: TObject);
var
  i: Integer;
begin
  if OpenDialog.Execute then
     begin
     OpenImgFile(OpenDialog.Filename);
     FileList.Clear;
     FileSearch(ExtractFilePath(OpenDialog.FileName), '*.*');
     i:=-1;
     repeat
     inc(i);
     until (OpenDialog.FileName=FileList.Items[i]) or (i>FileList.Count);
     Teller:=i;

     end;
end;

procedure TMain.OpenImgFile(Filename:String);
var
 SearchStr : String;
 Versie, i: Integer;
begin
  if Windows.ActivePage<>MeasuringTab then
     begin
     Windows.ActivePage:=MeasuringTab;
     WindowsChange(nil);
     end;
  If  NrofMicroscopes=0 then
     Showmessage('You have to create at least ONE microscope in preferences'+#10#13+
                 'and calibrate it');
  LoadBmpFromFile(Orgbmp,Filename);
  Display(Filename);
  SaveDialog.FileName:=FileName;
  // to be able to open images made with other camera's
  Versie:=0;

 // for i:=0 to 5 do
 //    RB[i].Visible:=False;
 // Objective.Caption:= 'No Microscope';
  if CommentList.Count<>0 then
     if CommentList.Strings[0] = '[MICAMUserComment]' then Versie:=1;
 // CheckScrollBar(UserMemo);
  if Versie=1 then GetInfoFromComment;
  Objective.Font.Color:=clRed;    // indicate that microscope might be incorrect
  ImgSourceIsFile:=True;
  if Versie=0 then
     begin
     Objective.Font.Color:=clBlack; // we don't know about settings inside image
     SelectedMic:=DefaultMic;
     SelectedObj:=DefaultObj;
     SetObjectives(SelectedMic,SelectedObj);
     ImgSourceIsFile:= False;
     end;
  FormResize(nil);
  ClearClick(nil);
end;


procedure TMain.GetInfoFromComment;

begin
   CommentMemo.Text:= ReadSystemComment;
   CheckScrollBar(CommentMemo);
   UserMemo.Text:=ReadUserComment;
   CheckScrollBar(UserMemo);
   ReadMicroscopeFromComment;
end;

procedure TMain.CheckScrollBar(Memo: TMemo);
begin
   if (Abs(Memo.Font.Height) + 4) * Memo.Lines.Count > Memo.Height then
      Memo.ScrollBars := ssVertical
   else
      Memo.ScrollBars := ssNone;
end;

procedure TMain.Next;
var
  Ext: String;
begin
  repeat
    Inc(Teller);
    if Teller=FileList.Count then
       if MessageDlg('Continue with first file in directory?', mtConfirmation, [mbYes, mbNo], 0) = mrYes
         then Teller:=0
         else begin Teller:=Teller-1; Exit; end;
    Ext:=lowerCase(ExtractFileExt(FileList.Items[Teller]));
    until (Ext='.bmp') or (Ext='.jpg') or(Ext='.png');
  OpenImgFile(FileList.Items[Teller]);
end;

procedure TMain.Previous;
var
 Ext: String;
begin
  repeat
    Dec(Teller);
    if Teller=-1 then
       if MessageDlg('Continue with last file in directory?', mtConfirmation, [mbYes, mbNo], 0) = mrYes
         then Teller:=Filelist.Count-1
         else begin Teller:=Teller+1; Exit; end;
    Ext:=lowerCase(ExtractFileExt(FileList.Items[Teller]));
    until (Ext='.bmp') or (Ext='.jpg') or(Ext='.png');
  OpenImgFile(FileList.Items[Teller]);
end;

procedure TMain.SaveImageAsClick(Sender: TObject);
begin
  FillComment;
  if ImgSourceIsFile=False then
     SaveDialog.FileName:=CreateFilename(0);
  if SaveDialog.Execute then
  begin
      SaveDialog.DefaultExt:='jpg';
      If SaveDialog.FilterIndex=1 then SaveDialog.DefaultExt:='bmp';
      If SaveDialog.FilterIndex=2 then SaveDialog.DefaultExt:='jpg';
      If SaveDialog.FilterIndex=3 then SaveDialog.DefaultExt:='png';
      StoreComment(UserMemo.Lines, CommentMemo.Lines);
      SaveBmpToFile(TekenBmp,SaveDialog.FileName);
      Display(SaveDialog.FileName);
   end;
end;

procedure TMain.SaveUseOptionsClick(Sender: TObject);
var
   Str:String;
begin
  if ImgSourceIsFile then SaveWithOptions.Caption:=Caption;
  SaveWithOptions.Snap:=False;
  if UserMemo.Text<>'' then SaveWithOptions.UserMemo.Text:= UserMemo.Text;
  if SaveWithOptions.Showmodal = mrCancel
        then Exit
        else Str:=SaveWithOptions.Directory.Text+'\'+SaveWithOptions.SugestedFileName.Text;
  Str:=Str+'.'+SaveWithOptions.ImageType.Text;
  if FileExists2(Str) then
     if MessageDlg('The file exists. Do you want to overwrite this file?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then Exit;
  LocationText1:=SaveWithoptions.LocationEdit1.Text ;
  LocationText2:=SaveWithoptions.LocationEdit2.Text ;
  StoreComment(SaveWithOptions.UserMemo.Lines,SaveWithOptions.CommentMemo.Lines);
  Display(Str);
  SaveBmpToFile(Tekenbmp,Str);
  if SaveWithOptions.Rename.Tag=1 then
     if DeleteFile(SaveWithOptions.Caption)=False then
        ShowMessage('Could not rename file. Created duplicate with new name');
  Caption:=Str;
  FillComment;
end;

procedure TMain.FileSearch(const PathName, FileName : string);
var Rec  : TSearchRec;
    Path : string;
begin
Path := IncludeTrailingBackslash(PathName);
if FindFirst(Path + FileName, faAnyFile - faDirectory, Rec) = 0 then
 try
   repeat
     FileList.Items.Add(Path + Rec.Name);
   until FindNext(Rec) <> 0;
 finally
   FindClose(Rec);
 end;
end;

procedure TMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin

  if Key=VK_Next then Next;
  if Key=VK_Prior then Previous;

  if Windows.ActivePage=PreviewTab then
     begin
     if Key=VK_F1 then SnapshotClick(nil);
     if Key=VK_F2 then SnapWithOptionsClick(nil);
     if Key=VK_F3 then ToMeasurementClick(nil);
     end;

  if Windows.ActivePage=MeasuringTab then
     begin
     if Key=VK_F4 then
        begin
        Windows.ActivePage:=PreviewTab;
        WindowsChange(nil);
        end;
     if Key=VK_F5 then OpenImageClick(nil);
     if Key=VK_F6 then SaveImageAsClick(nil);
     if Key=VK_F7 then SaveUseOptionsClick(nil);
     end;
end;


///////////////////////////////////////////////////////////////////
//
//        Menu Items
//
///////////////////////////////////////////////////////////////////

procedure TMain.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TMain.PreferencesClick(Sender: TObject);
begin
  PrefDlg:=TPrefDlg.Create(Self);    // Change preferences
  PrefDlg.ShowModal;
  PrefDlg.Free;
  SetObjectives(SelectedMic,SelectedObj);
  SetVidCapSize;
  FillComment;
  StartHWTimer;
end;

procedure TMain.ClearClick(Sender: TObject);
begin
 // Tekenbmp.Canvas.StretchDraw(Therect,Orgbmp);
 // Foto.Canvas.StretchDraw(Therect,Tekenbmp);
 FormResize(nil);
end;

procedure TMain.InsertScaleClick(Sender: TObject);
begin
  if TComponent(Sender).Name ='InsertScale1' then ScaleLength:=10;
  if TComponent(Sender).Name ='InsertScale2' then ScaleLength:=20;
  if TComponent(Sender).Name ='InsertScale3' then ScaleLength:=50;
  if TComponent(Sender).Name ='InsertScale4' then ScaleLength:=100;
  Foto.Cursor:=crHandPoint;
  MenuAction:=1;
end;

procedure TMain.InsertCopyright1Click(Sender: TObject);
begin
  Foto.Cursor:=crHandPoint;
  MenuAction:=2;
end;

procedure TMain.InsertLabelClick(Sender: TObject);
begin
  if InputQuery('Photo Label','Enter the Label text:',LabelString)=False then Exit;
  Foto.Cursor:=crHandPoint;
  MenuAction:=4;
end;

procedure TMain.CutSelectionClick(Sender: TObject);
var
 factor: Real;
begin
  X1:=-10;
  X2:=-10;
  Y1:=-10;
  Y2:=-10;
  if TComponent(Sender).Name ='CutSelection1' then
     begin
     if OrgBmp.Width<640 then
        begin ShowMessage('Not possible. Image is smaller than 640 x 480'); Exit; end;
     sw:=320;
     sh:=240;
     end;
  if TComponent(Sender).Name ='CutSelection2' then
     begin
     if OrgBmp.Width<800 then
        begin ShowMessage('Not possible. Image is smaller than 800 x 600'); Exit; end;
     sw:=400;
     sh:=300;
     end;
  if TComponent(Sender).Name ='CutSelection3' then
     begin
     if OrgBmp.Width<1024 then
        begin ShowMessage('Not possible. Image is smaller than 1024 x 768'); Exit; end;
     sw:=512;
     sh:=384;
     end;
  if TComponent(Sender).Name ='CutSelection4' then
     begin
     if OrgBmp.Width<1280 then
        begin ShowMessage('Not possible. Image is smaller than 1280 x 960'); Exit; end;
     sw:=640;
     sh:=480;
     end;
  factor:=OrgBmp.Width/Foto.Width;
  fsw:=Round(sw/factor);
  fsh:=Round(sh/factor);
  Foto.Cursor:=crCross;
  MoveSelect:=True;
  MenuAction:=3;
end;

procedure TMain.Help2Click(Sender: TObject);
begin
 HelpUnit.Help.WebBrowser1.Navigate(ProgDir+'micamhelp.html');
 HelpUnit.Help.ShowModal;
end;

procedure TMain.About1Click(Sender: TObject);
begin
 AboutDlg:= TAboutDlg.Create(self);
 AboutDlg.ShowModal;
 AboutDlg.Free;
end;


///////////////////////////////////////////////////////////////////
//
//        Calibration and Measurement  (Panel Items)
//
///////////////////////////////////////////////////////////////////



procedure TMain.FitToMeasurementWindowClick(Sender: TObject);
begin
 if WarningOnResize = True
    then WarningonResize:=not ShowWarning(
    'All measurements will be lost on resize!'+#10#13+
    'First set the right size, then perform measurements'+#10#13+
    'NOTE: The image will be saved in this format');
 FormResize(nil);
 ClearClick(nil);
 FillComment;
 Windows.SetFocus;
end;

///////////////////////////////////////////////////////////////////
//
//        Calibration and Measurement  (additional Routines)
//
///////////////////////////////////////////////////////////////////

procedure TMain.DoCalibrate;
var
  i : Integer;
  displacement, um, factor, Obj, N : Double;
  str, umstr : String;
begin
 if Calibrate.Checked = False then Exit;

 if ImgSourceIsFile then
        if OrgBmp.Width=0 then
           begin ShowMessage('Load image in measuring window first'); Exit; end;
     displacement:=sqrt(sqr(X2-X1) + sqr(Y2-Y1));
     if displacement=0 then displacement:=1;  // prevent division by zero
     str:='The distance between points was '+IntToStr(Round(displacement))+' pixels';
     umstr:='50';
     if InputQuery('Calibration', str+#13#10+'Enter the real distance in '+Units,umstr)=False then exit;
     um:=StrToFloat(umstr);
     if Units='mm' then um:=um*1000;
     CalValue:= um/displacement;
     if Orgbmp.Width<>Foto.Width then if OrgBmp.Width<>0 then CalValue:=CalValue*Foto.width/OrgBmp.Width;
     Obj:=Microscope[SelectedMic].Objective[SelectedObj].Obj;
     Microscope[SelectedMic].CaptureWidth:=OrgBmp.Width;
     If ImgsourceIsFile=False
        then
           begin
           if OrgBmp.Width<>0 then CalValue:=CalValue * VideoWidth/OrgBmp.Width;
           if CaptureDoubleSized=true then CalValue:=CalValue*2;
           end
        else
           begin
           CalValue:=CalValue* OrgBmp.Width/Microscope[SelectedMic].CaptureWidth;
           ImgCalValue:=CalValue;
           ImageObj:=Obj;
           end;
     for i:=0 to Microscope[SelectedMic].NrofObjectives-1 do
       if SelectedObj=i
          then Microscope[SelectedMic].Objective[i].Cal:=CalValue
          else
             if Microscope[SelectedMic].Objective[i].Cal =0 then
               begin
               N:=Microscope[SelectedMic].Objective[i].Obj;
               if N=0 then Factor:=0 else factor:=Obj/N;
               Microscope[SelectedMic].Objective[i].Cal:=-CalValue*factor;
               end;
     Action.Text:='';
     SetObjectives(SelectedMic, SelectedObj);
     CommentMemo.Lines[3]:=(Microscope[SelectedMic].Name+';  Obj: '+ FloatToStr(Microscope[SelectedMic].Objective[SelectedObj].Obj)
               + ' X;  um/pixel: '+FloatToStrF(CalValue,ffFixed,4,6));
end;

procedure TMain.DoMeasure;
var
  displacement, IW, D  : Real;
  ex, ey : Integer;
  distance: String;
begin
  displacement:=sqrt(sqr(X2-X1) + sqr(Y2-Y1));
  IW:= Microscope[SelectedMic].Objective[SelectedObj].Cal;
  if abs(IW)<0.0000000001 then ShowMessage('Not calibrated!');
  if IW<0 then ShowMessage('Using calculated calibration');
  CalValue:=abs(IW) *orgBmp.Width/Foto.Width;
  if ImgSourceIsFile=False
     then
        begin
        Calvalue:=Calvalue * Microscope[SelectedMic].CaptureWidth/VideoWidth;
        if CaptureDoubleSized=true then CalValue:=Calvalue/2;
        end
     else
        begin
        CalValue:=ImgCalValue*ImageObj/Abs(Microscope[-1].Objective[SelectedObj].Obj);
        CalValue:=CalValue* OrgBmp.Width / Foto.Width;
        end;
  D:=displacement*CalValue;

  if Units='mm'then D:=D/1000;
  if D>0.001 then Distance:=FloatToStrF(D,ffFixed,4,4);
  if D>0.01 then Distance:=FloatToStrF(D,ffFixed,4,3);
  if D>0.1 then Distance:=FloatToStrF(D,ffFixed,4,2);
  if D>5 then Distance:=FloatToStrF(D,ffFixed,4,1);
  if D>50 then Distance:=FloatToStrF(D,ffFixed,4,0);
  if D>1000 then Distance:=FloatToStrF(D,ffFixed,6,0);
  Action.Text:='Last Distance: '+Distance;
  ex:=X2; if X1>X2 then ex:=X1;
  ey:=Y2; if X1>X2 then ey:=Y1;
  with Tekenbmp.Canvas do
     begin
     Font.Size:=10;
     Font.Color:=clBlack;
     Brush.Color:=clWhite;
     TextOut(ex,ey,Distance);
     end;
  Foto.OnPaint(nil);
  end;


procedure TMain.FormResize(Sender: TObject);
var h,w : Integer;
      f : real;
begin
  if Windows.ActivePage=PreviewTab
  then
     begin
     VidCapBox.AutoScroll:=False;
     SetVidCapSize;
     VidCapBox.AutoScroll:=True;
     end
  else
     begin
     f := 800/600;  // default width-heigth ratio
     if Orgbmp.Width=0 then Exit;
     if orgbmp.Height>0 then f:=Orgbmp.Width/orgbmp.Height;
     h:=MeasureBox.Height;
     w:=MeasureBox.Width;
     if w/h > f then w:= Floor(f*h);
     if w/h <f then h:= Floor((w)/f);
     MeasureBox.AutoScroll:=False;
     Case FotoSize.ItemIndex of
     0: begin Foto.Width:=OrgBmp.Width; Foto.Height:=OrgBmp.Height; end;
     1: begin Foto.ClientWidth:=w-4; Foto.ClientHeight:=h-4; end;
     2: begin Foto.Width:=640; Foto.Height:=Round(Foto.Width/f); end;
     3: begin Foto.Width:=800; Foto.Height:=Round(Foto.Width/f); end;
     4: begin Foto.Width:=1024; Foto.Height:=Round(Foto.Width/f); end;
     5: begin Foto.Width:=1280; Foto.Height:=Round(Foto.Width/f); end;
      end; // case

     CalValue := Abs(Microscope[SelectedMic].Objective[SelectedObj].Cal);
     CalValue:=CalValue *OrgBmp.Width / Foto.Width;
     If ImgSourceIsFile= False
        then Calvalue:=Calvalue * Microscope[SelectedMic].CaptureWidth/VideoWidth
        else
           begin
           CalValue:=ImgCalValue*ImageObj/Microscope[SelectedMic].Objective[SelectedObj].Obj;
           if OrgBMp.Width<>0 then CalValue:=CalValue* Foto.Width / OrgBmp.Width;
           end;
     SetCalibrationText(CalValue);
     MeasureBox.AutoScroll:=True;
     Tekenbmp.Width:=Foto.Width;
     Tekenbmp.Height:=Foto.Height;
     TheRect:=Rect(0,0,Foto.Width,Foto.Height);
     Tekenbmp.Canvas.StretchDraw(TheRect,orgbmp);
     Foto.OnPaint(nil);
     end;
end;

procedure TMain.FotoMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
    if ssRight in Shift then CancelCutSelection;
    if MenuAction=0 then MeasureAndCalibrate(X,Y);
    if MenuAction=1 then PlaceScale(X,Y);
    if MenuAction=2 then PlaceString(CopyrightString,X,Y);
    if MenuAction=4 then PlaceString(LabelString,X,Y);
    if MenuAction=3 then CutSelection(X,Y);
end;

procedure TMain.FotoMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if MenuAction=1 then Exit;
  if MenuAction=2 then Exit;
  if MenuAction=4 then Exit;
  if MenuAction=3 then MoveSelection(X,Y);
  if Muis=Up then Exit;  // make sure that a proper mouse up is processed
  if MenuAction=0 then if ssLeft in Shift then DrawLine(X,Y);
end;

procedure TMain.FotoMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Foto.Cursor:=crDefault;
  if Muis=Up then Exit;
  Muis:=Up;  // make sure that only a mouse up after mouse down is processed
  if sqrt(sqr(X2-X1) + sqr(Y2-Y1))<4 then Exit;
  X2:=X;
  Y2:=Y;
  if Calibrate.Checked=True
     then
       with Tekenbmp.Canvas do
         begin
         Pen.Mode := pmXor;       //mode to remove last line
         Pen.Width:=MeasurePen.Width;
         Pen.Color:=MeasurePen.Color;
         if Pen.Color=clBlack then Pen.Mode:=pmNotXor;
         MoveTo(X1,Y1);
         LineTo(X2,Y2); //remove previous line
         DoCalibrate;
         end
     else
	   begin
	     Tekenbmp.Canvas.MoveTo(X1,Y1);
       Tekenbmp.Canvas.LineTo(X2,Y2);
       DoMeasure;
	   end;
  Calibrate.Checked:=False;
  Foto.OnPaint(nil);
end;

procedure TMain.DrawLine(X,Y : Integer);
begin
  with Tekenbmp.Canvas do
    begin
    Pen.Mode := pmXor;       //mode to remove last line
    Pen.Width:=MeasurePen.Width;
    Pen.Color:=MeasurePen.Color;
    if Pen.Color=clBlack then Pen.Mode:=pmNotXor;
    MoveTo(X1,Y1);
    LineTo(X2,Y2); //remove previous line
    MoveTo(X1,Y1);
    LineTo(X,Y); //Draw new line
    X2 := X;
    Y2 := Y;
    Pen.Mode := pmCopy;
    end;
    Foto.OnPaint(nil);
end;

procedure TMain.CancelCutSelection;
begin
   with Foto.Canvas do
        begin
        Pen.Mode:=pmXor;
        Pen.Width:=3;
        Pen.Color:=clBlack;
        if Pen.Color=clBlack then Pen.Mode:=pmNotXor;
        MoveTo(X1,Y1);
        LineTo(X1,Y2);
        LineTo(X2,Y2);
        LineTo(X2,Y1);
        LineTo(X1,Y1);
        MenuAction:=0;
        Foto.Cursor:=crDefault;
        Exit;
        end;
end;

procedure TMain.MeasureAndCalibrate(X, Y: Integer);
begin
  Muis:=Down;
  X1 := X;
  Y1 := Y;
  X2 := X;
  Y2 := Y;
  if Calibrate.Checked=True
     then Action.Text:='Calibrate'
     else Action.Text:='Measure';
end;


procedure TMain.PlaceScale(X, Y: Integer);
var
  umdiv, umincr : Integer;
  umpix, calvalue : Real;
  dx, dy : Integer;
  unitstr : String;
begin
  CalValue:=abs(Microscope[SelectedMic].Objective[SelectedObj].Cal) *orgBmp.Width/Foto.Width;
  if ImgSourceIsFile=False
     then
        begin
        Calvalue:=Calvalue * Microscope[SelectedMic].CaptureWidth/VideoWidth;
        if CaptureDoubleSized=true then CalValue:=Calvalue/2;
        end
     else
        begin
        CalValue:=ImgCalValue*ImageObj/Abs(Microscope[-1].Objective[SelectedObj].Obj);
        CalValue:=CalValue* OrgBmp.Width / Foto.Width;
        end;
  umpix:=CalValue;
  if units='mm' then umpix:= umpix/1000;
  MenuAction:=0;
  if umpix<>0 then
  begin
  umdiv:= 0;
  umincr:= 10;
  if ScaleLength=100 then umincr:=20;
  with Tekenbmp.Canvas do
    begin
    Pen.Color:=ScalePen.Color;
    Pen.Width:=ScalePen.Width;
    Pen.Mode:=pmCopy;
    Font.Name:='MS Sans Serif';
    Font.Size:=10;
    Font.Color:=ScalePen.Color;
    Font.Style:=[fsBold];
    Brush.Color:=clWhite;
    Brush.Style:=bsClear;
    unitstr:=Units;
    if units='um' then unitstr:= chr($B5)+'m';
    if X>Foto.Width div 2 then X:=Round(X-ScaleLength/umpix);
    MoveTo(X,Y);
    LineTo(Round(X+ScaleLength/umpix),Y);
    repeat
      MoveTo(Round(X+umdiv/umpix),Y-2*ScalePen.Width);
      LineTo(Round(X+umdiv/umpix),Y+2*ScalePen.Width);
      umdiv:=umdiv+umincr;
    until umdiv>ScaleLength;
    dx:=Round((ScaleLength/umpix)/2-(TextWidth(IntToStr(ScaleLength)+'um'))/2);
    dy:=TextHeight(IntToStr(ScaleLength)+' '+unitstr)+5+ScalePen.Width div 2;
    TextOut(X+dx,Y-dy,IntToStr(ScaleLength)+' '+unitstr);
    end;
  end;
  Foto.OnPaint(nil);
end;

procedure TMain.PlaceString(Str: String;X, Y: Integer);
begin
  MenuAction:=0;
  If Pos('(c)',Str)<>0 then Str:=StuffString(Str,Pos('(c)',Str),3,chr($A9)); // replace (c) by copyright symbol
  if Str<>'' then
    begin
       with Tekenbmp.Canvas do
         begin
         Font.Size:=CopyrightFont.Size;
         Font.Name:=CopyrightFont.Name;
         Font.Color:=CopyrightFont.Color;
         Font.Style:=CopyrightFont.Style;
         Brush.Color:=clWhite;
         Brush.Style:=bsClear;
         if X>Foto.Width div 2 then X:=X-TextWidth(Str);
         if Y>Foto.Width div 2 then Y:=Y-TextHeight(Str);
         TextOut(X,Y,Str);
         end;
    end;
   Foto.OnPaint(nil);
end;

procedure TMain.MoveSelection(X, Y: Integer);
begin
  with Foto.Canvas do
  begin
  Pen.Mode:=pmNotXor;
  Pen.Color:=clBlack;
  Pen.Width:=3;
  MoveTo(X1,Y1);
  LineTo(X1,Y2);
  LineTo(X2,Y2);
  LineTo(X2,Y1);
  LineTo(X1,Y1);
  X1:=X-fsw;
  Y1:=Y-fsh;
  X2:=X+fsw;
  if X1<0 then begin X1:=0;X2:=X1+2*fsw;end;
  if X2 > Foto.Width then begin X2:=Foto.Width;X1:=X2-2*fsw;end;
  Y2:=Y+fsh;
  if Y1<0 then begin Y1:=0; Y2:=Y1+2*fsh; end;
  if Y2> Foto.Height then begin Y2:=Foto.Height;Y1:=Y2-2*fsh;end;
  MoveTo(X1,Y1);
  LineTo(X1,Y2);
  LineTo(X2,Y2);
  LineTo(X2,Y1);
  LineTo(X1,Y1);
  end;
end;

procedure TMain.CutSelection(X, Y: Integer);
var
 factor : Real;
 Str : String;
begin
  If MoveSelect=False then Exit;
  factor:=OrgBmp.Width/Foto.Width;
  with Foto.Canvas do
  begin
  Pen.Mode:=pmXor;
  Pen.Width:=3;
  Pen.Color:=clBlack;
  if Pen.Color=clBlack then Pen.Mode:=pmNotXor;
  MoveTo(X1,Y1);
  LineTo(X1,Y2);
  LineTo(X2,Y2);
  LineTo(X2,Y1);
  LineTo(X1,Y1);
  MoveSelect:=False;
  MenuAction:=0;
  OrgBmp.Canvas.CopyRect(
    Rect(0,0,2*sw,2*sh),
    OrgBmp.Canvas,
    Rect(Round(factor*X1),Round(factor*Y1),Round(factor*(X1)+sw+sw),Round(factor*(Y1)+sh+sh)));
  Orgbmp.Width:=2*sw;
  OrgBmp.Height:=2*sh;
  end;
 TekenBmp.Canvas.CopyRect(TheRect,Orgbmp.Canvas,TheRect);
 Foto.Cursor:=crDefault;
 Str:=CreateFilename(0);
 SaveDialog.FileName:=Str;
 Display(Str);
 FillComment;
 ImgSourceIsFile:=False;
 FormResize(nil);
end;

procedure TMain.FotoPaint(Sender: TObject);
begin
 Foto.Canvas.CopyRect(TheRect,Tekenbmp.Canvas,TheRect);
end;

procedure TMain.DoubleSize(Src:TBitmap;var Dst:TBitmap);
type TWa = array [0..3000] OF TRGBTriple;
     PWa = ^TWa;
var
 bmp0 : TBitmap;
 i, j : Integer;
 po1, po2, p1, p2 : PWa;
begin
 bmp0:=Tbitmap.Create;
 bmp0.Width:=VidCap.Width*2;
 bmp0.Height:=VidCap.Height*2;

 bmp0.PixelFormat := pf24bit;
 Src.PixelFormat := pf24bit;
 Src.Width:=VidCap.Width;
 Src.Height:=VidCap.Height;

BEGIN
  FOR j := 0 TO Src.Height-2 DO
    begin
    p1 := Src.scanline[j];
    p2 := Src.scanline[j+1];

    po1:=bmp0.ScanLine[j*2];
    po2:=bmp0.ScanLine[(j*2)+1];
    FOR i := 0 TO Src.Width-2  DO
      begin
      po1^[i*2] := p1^[i];         // linksboven

      po1^[(i*2)+1].rgbtBlue := (p1^[i].rgbtBlue+p1^[i+1].rgbtBlue)div 2;  //rechtsboven
      po1^[(i*2)+1].rgbtGreen := (p1^[i].rgbtGreen+p1^[i+1].rgbtGreen)div 2;
      po1^[(i*2)+1].rgbtRed := (p1^[i].rgbtRed+p1^[i+1].rgbtRed) div 2;

      po2^[(i*2)].rgbtBlue := (p1^[i].rgbtBlue+p2^[i].rgbtBlue)div 2;  //linksonder
      po2^[(i*2)].rgbtGreen := (p1^[i].rgbtGreen+p2^[i].rgbtGreen)div 2;
      po2^[(i*2)].rgbtRed := (p1^[i].rgbtRed+p2^[i].rgbtRed) div 2;

      po2^[(i*2)+1].rgbtBlue := (p1^[i].rgbtBlue+p2^[i].rgbtBlue+p1^[i+1].rgbtBlue+p2^[i+1].rgbtBlue)div 4;  //rechtsonder
      po2^[(i*2)+1].rgbtGreen:= (p1^[i].rgbtGreen+p2^[i].rgbtGreen+p1^[i+1].rgbtGreen+p2^[i+1].rgbtGreen)div 4;
      po2^[(i*2)+1].rgbtRed  := (p1^[i].rgbtRed+p2^[i].rgbtRed+p1^[i+1].rgbtRed+p2^[i+1].rgbtRed) div 4;
      end;
    end;
    Dst.Assign(bmp0);
    Dst.Width:=VidCap.Width*2;
    Dst.Height:=VidCap.Height*2;
END;
 bmp0.Free;
end;

procedure TMain.StackIt(var Dst:TBitmap);
type TWa = array [0..3000] OF TRGBTriple;
     PWa = ^TWa;
var
 bmp0, bmp1 : TBitmap;
 i, j, t, nr : Integer;
 po, p1 : PWa;
begin
 bmp0:=Tbitmap.Create;
 bmp0.Width:=VideoWidth;
 bmp0.Height:=VideoHeight;

 bmp1:=Tbitmap.Create;
 bmp1.Width:=bmp0.Width; //.VCapMode.Width;
 bmp1.Height:=bmp0.Height; //.VCapMode.Height;

 // clear image
  FOR j := 0 TO bmp1.Height-1 DO
    FOR i := 0 TO bmp1.Width-1  DO
       For t:= 1 to 3 do
          MemImage[i,j,t]:=0;

 nr := StrToInt(NrBox.Text);
try
For t:=1 to nr do
begin
 Counter.Caption:=IntToStr(nr-t);

 Application.ProcessMessages;
 CaptureFrame;
 GetCapturedFrame(bmp1);
 bmp0.PixelFormat := pf24bit;
 bmp1.PixelFormat := pf24bit;

 FOR j := 0 TO bmp1.Height-1 DO
    begin
    p1 := bmp1.scanline[j];
    FOR i := 0 TO bmp1.Width-1  DO
      begin
        MemImage[i,j,1]:=MemImage[i,j,1]+(p1^[i].rgbtBlue);
        MemImage[i,j,2]:=MemImage[i,j,2]+(p1^[i].rgbtGreen);
        MemImage[i,j,3]:=MemImage[i,j,3]+(p1^[i].rgbtRed);
      end;
    end;
  end;   //  frame stacked

  FOR j := 0 TO bmp1.Height-1 DO
    begin
    po := bmp0.ScanLine[j];
    FOR i := 0 TO bmp1.Width-1  DO
       begin
          po^[i].rgbtBlue := IntToByte(MemImage[i,j,1] div nr);
          po^[i].rgbtGreen := IntToByte(MemImage[i,j,2] div nr);
          po^[i].rgbtRed := IntToByte(MemImage[i,j,3] div nr);
       end;
    end;
  Dst.Assign(bmp0);
  Dst.Width:=bmp0.Width;
  Dst.Height:=bmp0.Width;
finally
 bmp0.Free;
 bmp1.Free;
end;
end;


procedure TMain.CBNoiseReductionClick(Sender: TObject);
begin
 if CBNoiseReduction.Checked=True
    then begin
         NrBox.Visible:=True;
         Label3.Visible:=True;
         Counter.Visible:=True;
         end
    else begin
         NrBox.Visible:=False;
         Label3.Visible:=False;
         Counter.Visible:=False;
         end;
end;

procedure TMain.PanelWindowProc(var Msg: TMessage) ;
begin
   if Msg.Msg = WM_DROPFILES then
     PanelImageDrop(TWMDROPFILES(Msg))
   else
     originalPanelWindowProc(Msg) ;
end; (*PanelWindowProc*)

procedure TMain.PanelImageDrop(var Msg: TWMDROPFILES) ;
var
   numFiles : longInt;
   buffer : array[0..MAX_PATH] of char;
   i : Integer;
begin
   numFiles := DragQueryFile(Msg.Drop, $FFFFFFFF, nil, 0) ;
   if numFiles > 1 then
   begin
     ShowMessage('You can drop only one image file at a time!') ;
   end
   else
   begin
     DragQueryFile(Msg.Drop, 0, @buffer, sizeof(buffer)) ;
     try
       OpenImgFile(buffer);
       FileList.Clear;
       FileSearch(ExtractFilePath(buffer), '*.*');
       i:=-1;
       repeat
        inc(i);
       until (buffer=FileList.Items[i]) or (i>FileList.Count);
       Teller:=i;
     except
       on EInvalidGraphic do ShowMessage('Unsupported image file, or not an image!') ;
     end;
   end;
end; (*PanelImageDrop*)



procedure TMain.TimerTimer(Sender: TObject);
begin
  CommentMemo.Visible:=False;
  SystemCommentLabel.Visible:=False;
  Timer.Enabled:=False;
end;

procedure TMain.Display(Filename: String);
begin
  Caption:=FileName;
  Statusbar.Panels[1].Text:=ExtractFileName(Filename);
end;

procedure TMain.FillComment;
var
  S, U : TStringlist;
begin
  S:=TStringlist.Create;
  U:=TStringlist.Create;
  S.Add('');
  S.Add('');
  S.Add('');
  S.Add('');
  S.Strings[0]:=(LocationText1);
  S.Strings[1]:=(LocationText2);
  S.Strings[2]:=('SampleDate = '+DateToStr(sampleDate));
  S.Strings[3]:=(Microscope[SelectedMic].Name+';  Obj: '+ FloatToStr(Microscope[SelectedMic].Objective[SelectedObj].Obj)
               + ' X;  um/pixel: '+FloatToStrF(CalValue,ffFixed,4,6));
  StoreComment(U, S);
  CommentMemo.Text:=S.Text;
  S.Free;
  U.Free;
end;

procedure TMain.StoreComment(UsrStr,SystemStr: TStrings);
var
  j : Integer;
begin
  CommentList.Clear;
  CommentList.Add('[MICAMUserComment]');
  CommentList.AddStrings(UsrStr);
  CommentList.Add('[MICAMSystemComment]');
  CommentList.AddStrings(SystemStr);
  CommentList.Add('[MICAMHiddenComment]');
  CommentList.Add('Revision = '+ IntToStr(VERSION*1000+MAJOR*100+MINOR));
  CommentList.Add('Device = '+VideoDevice);
  CommentList.Add('Magnification = '+FloatToStr(Microscope[SelectedMic].Objective[SelectedObj].Obj)+' X');
  CommentList.Add('Calibration = '+FloatToStrF(CalValue,ffFixed,4,6));
  CommentList.Add('Microscope = '+Microscope[SelectedMic].Name);
  CommentList.Add('CaptureWidth = '+IntToStr(Microscope[SelectedMic].CaptureWidth));
  CommentList.Add('NrOfObjectives = '+IntToStr(Microscope[SelectedMic].NrofObjectives));
  CommentList.Add('ImageObj = '+IntToStr(SelectedObj));
  if Microscope[SelectedMic].NrofObjectives >0 then
  for j:=0 to Microscope[SelectedMic].NrofObjectives-1 do
   begin
    CommentList.Add('obj'+FloatToStr(j)+' = '+FloatToStr(Microscope[SelectedMic].Objective[j].Obj));
    CommentList.Add('cal'+FloatToStr(j)+' = '+FloatToStr(Microscope[SelectedMic].Objective[j].Cal));
   end;
end;

function TMain.ReadUserComment: String;
 var
   i : Integer;
   S : TStringlist;
   str : String;
begin
   S:=TStringlist.Create;
   S.Text:='';
   str:= 'MICAMSystemComment';
   if CommentList.Strings[0]='[MICAMUserComment]' then str:= '[MICAMSystemComment]';
   i:=1;
      while (CommentList.Strings[i]<>str) and (i<CommentList.Count-1)do
        begin
        S.Add(CommentList.Strings[i]);
        Inc(i)
       end;
   Result:=S.Text;
   S.Free;
end;

function TMain.ReadSystemComment: String;
 var
   i : Integer;
   S : TStringlist;
   str : String;
begin
   S:=TStringlist.Create;
   S.Text:='';
   str:= 'MICAMSystemComment';
   if CommentList.Strings[0]='[MICAMUserComment]' then str:= '[MICAMSystemComment]';
   i:=1;
      while (CommentList.Strings[i]<>str) and (i<CommentList.Count-1)do
        begin
        Inc(i)
       end;
   inc(i);
   str:= 'MICAMHiddenComment';
   if CommentList.Strings[0]='[MICAMUserComment]' then str:= '[MICAMHiddenComment]';
      while (CommentList.Strings[i]<>str) and (i<CommentList.Count-1)do
        begin
        S.Add(CommentList.Strings[i]);
        Inc(i)
       end;
   Result:=S.Text;
   S.Free;
end;

procedure TMain.ReadMicroscopeFromComment;
var
 j : integer;
 versie, IObj : Integer;
begin
  try
  Microscope[-1].Name:= FindInLines('microscope',Commentlist);
  Microscope[-1].CaptureWidth:=StrToInt(FindInLines('CaptureWidth',Commentlist));
  Microscope[-1].NrOfObjectives:=StrToInt(FindInLines('NrOfObjectives',Commentlist));
  IObj:= StrToInt(FindInLines('ImageObj',Commentlist));
  ImgCalValue:=StrToFloat(FindInLines('calibration',Commentlist));
  for j:=0 to Microscope[-1].NrofObjectives-1 do
    begin
    Microscope[-1].Objective[j].Obj:=StrToFloat(FindInLines('obj'+IntToStr(j),Commentlist));
    Microscope[-1].Objective[j].Cal:=StrToFloat(FindInLines('cal'+IntToStr(j),Commentlist));
    ImageObj:= Microscope[-1].Objective[IObj].Obj
    end;
  except Exit;
  end;
  SelectedMic:=-1;
  SelectedObj:=IObj;
  SetObjectives(-1,IObj);
end;

procedure TMain.EditCalibrationValueClick(Sender: TObject);
var
 ump : Real;
 Str : String;
begin
  if OrgBmp.Width=0 then
    begin ShowMessage('Load image in measuring window first'); Exit; end;

  Str:=Inputbox('Edit Calibration Value','Change the '+Units+'/pixel value for the selected objective', FloattoStr(CalValue));
  ump:=CalValue;
  if Main.CaptureDoubleSized then ump:=ump*2;
  Microscope[SelectedMic].Objective[SelectedObj].Cal:=ump;
  Microscope[SelectedMic].CaptureWidth:=OrgBmp.Width;
  SetObjectives(SelectedMic, SelectedObj);
end;

procedure TMain.SearchClick(Sender: TObject);
begin
  SearchForm.ShowModal;
end;

procedure TMain.StartHWTimer;
begin
  HWTimer.Enabled:=UseHWTimer;
  if UseHWTimer= False then Exit;
  RTS((Main.PoortHigh and 1)=1);
  DTR((Main.PoortHigh and 2)=2);
end;

procedure TMain.HWTimerTimer(Sender: TObject);
begin
 Case PoortLine of
   0 : NewTrig:=CTS;
   1 : NewTrig:=DSR;
   2 : NewTrig:=DCD;
   3 : NewTrig:=RI;
  end;
  if NewTrig= OldTrig then Exit;
  OldTrig:=NewTrig;
  if NewTrig = PoortActive then SnapshotClick(nil);
end;

procedure TMain.DelayTimer(Sender: TObject);
begin
  Delay.Enabled:=False;
if VideoDevice<>'' then
    begin
    VidCap.SaveGraph(config);
    config.VCapSource:= VideoDevice;
    config.VComp:=Compressor;
    config.Vcapmode.Width:=VideoWidth;
    config.VCapMode.Height:=VideoHeight;
    VidCap.RestoreGraph(config);
    FrameCapture.Enabled:=True;
    end;
end;

procedure TMain.SetCalibrationText(CalValue: Real);
begin
  if Units='mm' then CalValue:=CalValue/1000;
  Calibration.Text:=FloatToStrF(CalValue,ffFixed,4,5);
  CalLabel.Caption:=Units+' / pixel';
end;


procedure TMain.WindowsMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  WindowsChange(nil);
end;

end.
