unit MicamUtils;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Inifiles,
  Clipbrd, FileCtrl, JPeg, PNGImage, JPGComment, Dialogs, Forms, StdCtrls;
  
function IntToByte(i:Integer):Byte;
function FileExists2(const FileName: string): boolean;
procedure CreateValidFileName (var Filename:String);
procedure SavePrefsToIni;
procedure LoadPrefsFromIni;
procedure SaveCameraInfo(Camera:String);
procedure LoadCameraInfo(Camera:String);
procedure SaveBmpToFile(Bmp: TBitmap;FileName:String);
procedure LoadBmpFromFile(var bmp:TBitmap;FileName:String);
function FindInLines(Str:String;Lines:TStrings):String;
function ShowWarning(Msg:String): Boolean;

const
ForbiddenChars  : set of Char = ['<', '>', '|', '"', '\', '/', ':', '*', '?'];

implementation

uses
  MICAMMainUnit, SaveOptions;

function IntToByte(i:Integer):Byte;
begin
  if      i>255 then Result:=255
  else if i<0   then Result:=0
  else               Result:=i;
end;

function FileExists2(const FileName: string): boolean;
var
  Handle: THandle;
  FindData: TWin32FindData;
begin
  Handle := FindFirstFile(PChar(FileName), FindData);
  if Handle <> INVALID_HANDLE_VALUE then
    begin
      Windows.FindClose(Handle);
      if (FindData.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY)=0 then
        Result:=true
      else
        Result:=false;
    end
  else Result:=false;
end;

procedure CreateValidFileName (var Filename:String);
var
  I: integer;
begin
  for I := 1 to Length(Filename) do
      if(Filename[I] in ForbiddenChars) then FileName[I]:='_';
end;

procedure SavePrefsToIni;
var
  ini:TInifile;
  i,j : integer;
  str1, str2, str3, str4 : String;
begin
With Main do
 begin
 if FileExists2(Progdir+'micam.ini') then
    DeleteFile(Progdir+'micam.ini');
 ini:= TInifile.Create(Progdir+'micam.ini');

 ini.WriteInteger('Revision','MajorMinor',VERSION*1000+MAJOR*100+MINOR);

 ini.WriteInteger('Window','Top',Top);
 ini.WriteInteger('Window','Left',Left);
 ini.WriteInteger('Window','Width',Width);
 ini.WriteInteger('Window','Height',Height);
 Case WindowState of
   wsNormal:     ini.WriteInteger('Window','State',0);
   wsMaximized: ini.WriteInteger('Window','State',1);
   wsMinimized:  ini.WriteInteger('Window','State',2);
   end;

 ini.WriteBool('Device','UseDeviceDefaultSettings', UseDeviceDefaultSettings);
 ini.WriteString('Device', 'Video', VideoDevice);
 ini.WriteBool('Device','NoDevice', NoDevice);
 SaveCameraInfo(VideoDevice);
 ini.WriteInteger('Video', 'Width',VideoWidth );
 ini.WriteInteger('Video', 'Height',VideoHeight );
 ini.WriteInteger('Video','SnapSize', SnapSize);
 ini.WriteBool('Video','FitToPreviewWindow', FitToPreviewWindow.Checked);
 ini.WriteInteger('Video', 'FotoSize', FotoSize.ItemIndex );
 ini.WriteBool('Video','UseFrameRate', VidCap.UseFrameRate);
 ini.WriteFloat('Video','FrameRate',Vidcap.FrameRate);
 //ini.WriteBool('Video','UseClock', VidCap.UseClock);
 ini.WriteBool('Video','UseTimeLimit', VidCap.UseTimeLimit);
 ini.WriteInteger('Video','TimeLimit',Vidcap.TimeLimit);

 If NrOfMicroscopes=1 then DefaultMic:=0;
 ini.WriteInteger('Microscopes', 'DefaultMic', DefaultMic);
 ini.WriteInteger('Microscopes', 'DefaultObj', DefaultObj);

 ini.WriteString('Directories','AVIDir',AVIdir);
 ini.WriteString('Directories','FrameDir',Framedir);
 ini.WriteString('Location','Location',LocationText1);
 ini.WriteString('Location','Coordinates',LocationText2);

 ini.WriteInteger('Files','ImageExtention',ImageKind);
 ini.WriteInteger('Files','CaptureFileType',CaptureFileType);
 ini.WriteInteger('Files','CaptureNr',CaptureNr);
 ini.WriteString('Files','CaptureFileName',CaptureName);
 ini.WriteBool('Files','CaptureDoubleSized',CaptureDoubleSized);
 ini.WriteBool('Files','CaptureMaxSize', FitToPreviewWindow.Checked);
 ini.WriteBool('Files','SnapshotWarning', SnapshotWarning);
 ini.WriteBool('Files','BMPcomment', BMPcomment);

 ini.WriteBool('Sound','SoundEnabled', SoundEnabled);

 ini.WriteBool('Hardware','UseHardwareTrigger', UseHWTimer);
 ini.WriteInteger('HardWare','PortNumber',Poortnr);
 ini.WriteInteger('HardWare','PortLine',PoortLine);
 ini.WriteInteger('HardWare','PortHigh',PoortHigh);
 ini.WriteBool('Hardware','PortActive', PoortActive);

 ini.WriteInteger('MeasuringWindow','MeasurePenWidth',MeasurePen.Width);
 ini.WriteInteger('MeasuringWindow','MeasurePenColor',MeasurePen.Color);
 ini.WriteInteger('MeasuringWindow','ScalePenWidth',ScalePen.Width);
 ini.WriteInteger('MeasuringWindow','ScalePenColor',ScalePen.Color);
 ini.WriteString('MeasuringWindow','Units',Units);

 ini.WriteString('Copyright','Text',CopyRightString);
 Ini.WriteString('Copyright','FontName',CopyrightFont.Name);
 ini.WriteInteger('Copyright','Color',CopyrightFont.Color);
 ini.WriteInteger('Copyright','Size',CopyrightFont.Size);
 If CopyrightFont.Style=[] then Ini.WriteInteger('Copyright','Style',0);
 If CopyrightFont.Style=[fsBold] then Ini.WriteInteger('Copyright','Style',1);
 If CopyrightFont.Style=[fsItalic] then Ini.WriteInteger('Copyright','Style',2);
 If CopyrightFont.Style=[fsUnderline] then Ini.WriteInteger('Copyright','Style',3);
 If CopyrightFont.Style=[fsStrikeout] then Ini.WriteInteger('Copyright','Style',4);

 ini.WriteBool('Warnings','WarningOnResize', WarningOnResize);
 ini.WriteBool('Warnings','WarningOnAVI', WarningOnAVI);

 ini.WriteInteger('Microscopes', 'NrOfMicroscopes', NrofMicroscopes);

 if NRofMicroscopes<>0 then
 begin
 for i:=0 to NrOfMicroscopes-1 do
   begin
   str3:='Microscope'+IntToStr(i);
   ini.WriteString(str3,'Name'+IntToStr(i), Microscope[i].Name);
   ini.WriteInteger(str3,'CaptureWidth',Microscope[i].CaptureWidth);
   ini.WriteInteger(str3,'NrofObjectives',Microscope[i].NrOfObjectives);
   if Microscope[i].NrofObjectives <>0 then
      begin
         for j:=0 to Microscope[i].NrofObjectives-1 do
             begin
             str1:='Obj'+IntToStr(j);
             str2:='Cal'+IntToStr(j);
             str4:='ObjStr'+IntToStr(j);
             ini.WriteFloat(str3,str1, Microscope[i].Objective[j].Obj);
             ini.WriteString(str3,str4, Microscope[i].Objective[j].ObjStr);
             ini.WriteFloat(str3,str2, Microscope[i].Objective[j].Cal);
             end;
      end;
   end;
 end;
 ini.free;
 end;
end;

procedure LoadPrefsFromIni;
var ini:TInifile;
     i,j  :Integer;
     str1, str2, str3, str4, dummy : String;
     Revision, State: Integer;
begin
 DateSeparator := '-';
 ShortDateFormat := 'yyy/mm/dd';
 With Main do
 begin
 ini:= TInifile.Create(Progdir+'micam.ini');

 Revision:=ini.ReadInteger('Revision','MajorMinor',0);
 if Revision=0 then
    Revision:=ini.ReadInteger('Revision','Major',0); // compatibility with older versions

 Top:=ini.ReadInteger('Window','Width', 0);
 Left:=ini.ReadInteger('Window','Width', 0);
 Width:=ini.ReadInteger('Window','Width', 815);
 Height:=ini.ReadInteger('Window','Height', 655);
 State:=ini.ReadInteger('Window','State', 0);
 Case State of
   0:     WindowState:=wsNormal;
   1:     WindowState:=wsMaximized;
   2:     WindowState:=wsMinimized;
   else   WindowState:=wsNormal;
   end;


 UseDeviceDefaultSettings:=ini.ReadBool('Device','UseDeviceDefaultSettings', False );
 VideoDevice:=ini.ReadString('Device', 'Video','' );
 NoDevice:=ini.ReadBool('Device','NoDevice', False);
 
 if UseDeviceDefaultSettings=True then LoadCameraInfo(VideoDevice)
    else
    begin
    VideoWidth:=ini.ReadInteger('Video', 'Width',640 );
    VideoHeight:=ini.ReadInteger('Video', 'Height',480 );
    SnapSize:=ini.ReadInteger('Video','SnapSize', 0 );
    FitToPreviewWindow.Checked:=ini.ReadBool('Video','FitToPreviewWindow', False );
    FotoSize.ItemIndex:=ini.ReadInteger('Video', 'FotoSize', 0 );
    VidCap.UseFrameRate:=ini.ReadBool('Video','UseFrameRate', False);
    Vidcap.FrameRate:=ini.ReadInteger('Video','FrameRate',25);
   // VidCap.UseClock:=ini.ReadBool('Video','UseClock', False);
    VidCap.UseTimeLimit:=ini.ReadBool('Video','UseTimeLimit', True );
    Vidcap.TimeLimit:=ini.ReadInteger('Video','TimeLimit',30);

    DefaultMic:=ini.ReadInteger('Microscopes', 'DefaultMic',0);
    DefaultObj:=ini.ReadInteger('Microscopes', 'DefaultObj',0);
    SelectedMic:=DefaultMic;
    SelectedObj:=DefaultObj;
    end;

 AVIDir:=ini.ReadString('Directories','AVIDir',Main.Progdir+'\Capture');
 FrameDir:=ini.ReadString('Directories','FrameDir',Main.Progdir+'Capture');
 LocationText1:=ini.ReadString('Location','Location','');
 LocationText2:=ini.ReadString('Location','Coordinates','');
 ImageKind:=ini.ReadInteger('Files','ImageExtention',1);
 CaptureFileType:=ini.ReadInteger('Files','CaptureFileType',0);
 CaptureNr:=ini.ReadInteger('Files','CaptureNr',0);
 CaptureName:=ini.ReadString('Files','CaptureFileName','IMG');
 CaptureDoubleSized:=ini.ReadBool('Files','CaptureDoubleSized', False);
 FitToPreviewWindow.Checked:=ini.ReadBool('Files','CaptureMaxSize', False);
 SnapshotWarning:=ini.ReadBool('Files','SnapshotWarning', False);
 BMPcomment:=ini.ReadBool('Files','BMPcomment', True);

 SoundEnabled:=ini.ReadBool('Sound','SoundEnabled', False);

 UseHWTimer:=ini.ReadBool('Hardware','UseHardwareTrigger', False);
 Poortnr:=ini.ReadInteger('HardWare','PortNumber', 0);
 PoortLine:=ini.ReadInteger('HardWare','PortLine', 0);
 PoortHigh:=ini.ReadInteger('HardWare','PortHigh', 0);
 PoortActive:=ini.ReadBool('Hardware','PortActive', False);

 MeasurePen.Width:=ini.ReadInteger('MeasuringWindow','MeasurePenWidth',1);
 MeasurePen.Color:=ini.ReadInteger('MeasuringWindow','MeasurePenColor',clBlack);
 ScalePen.Width:=ini.ReadInteger('MeasuringWindow','ScalePenWidth',1);
 ScalePen.Color:=ini.ReadInteger('MeasuringWindow','ScalePenColor',clBlack);
 Units:=ini.ReadString('MeasuringWindow','Units','um');

 CopyRightString:=ini.ReadString('Copyright','Text','Made with MICAM');
 CopyrightFont.Name:=Ini.ReadString('Copyright','FontName','MS Sans Serif');
 CopyrightFont.Color:=ini.ReadInteger('Copyright','Color',clBlack);
 CopyrightFont.Size:=ini.ReadInteger('Copyright','Size',12);
 Case Ini.ReadInteger('Copyright','Style',1) of
     0 : CopyrightFont.Style:=[];
     1 : CopyrightFont.Style:=[fsBold];
     2 : CopyrightFont.Style:=[fsItalic];
     3 : CopyrightFont.Style:=[fsUnderline];
     4 : CopyrightFont.Style:=[fsStrikeout];
     else
       CopyrightFont.Style:=[];
  end;{Case}
 WarningOnResize:=ini.ReadBool('Warnings','WarningOnResize', True);
 WarningOnAVI:=ini.ReadBool('Warnings','WarningOnAVI', True);

 NrofMicroscopes:=ini.ReadInteger('Microscopes', 'NrOfMicroscopes',0);

 if NRofMicroscopes<>0 then
 begin
 for i:=0 to NrOfMicroscopes-1 do
   begin
   str3:='Microscope'+IntToStr(i);
   Microscope[i].Name:=ini.ReadString(str3,'Name'+IntToStr(i),'NoName');
   Microscope[i].CaptureWidth:=ini.ReadInteger(str3,'CaptureWidth',0);
   if Revision >= 900 then Microscope[i].NrofObjectives:=ini.ReadInteger(str3,'NrOfObjectives',0)
                      else Microscope[i].NrofObjectives:=6;
   if Microscope[i].NrofObjectives <>0 then
      begin
         for j:=0 to Microscope[i].NrofObjectives-1 do
            begin
            str1:='Obj'+IntToStr(j);
            str4:='ObjStr'+IntToStr(j);
            str2:='Cal'+IntToStr(j);
            if Revision < 900 then dummy:=ini.ReadString(str3,'ObjText'+IntToStr(j), '');
            Microscope[i].Objective[j].Obj:=ini.ReadFloat(str3,str1, 0);
            Microscope[i].Objective[j].ObjStr:=ini.ReadString(str3,str4, FloatToStr(Microscope[i].Objective[j].Obj)+' X');
            Microscope[i].Objective[j].Cal:=ini.ReadFloat(str3,str2, 0);
            if (Revision<900) and (Microscope[i].Objective[j].Obj<>0) then Microscope[i].Objective[j].Cal:=
                 Microscope[i].Objective[j].Cal/Microscope[i].CaptureWidth/Microscope[i].Objective[j].Obj;
            end;
      end;
   end;
 end;
 ini.free;
 end;
  Main.InsertScale1.Caption:='Insert Scale 10 '+Main.Units;
  Main.InsertScale2.Caption:='Insert Scale 20 '+Main.Units;
  Main.InsertScale3.Caption:='Insert Scale 50 '+Main.Units;
  Main.InsertScale4.Caption:='Insert Scale 100 '+Main.Units;
  Main.CalLabel.Caption:=Main.Units+' / pixel';
end;

procedure SaveCameraInfo(Camera:String);
var
  dat:TInifile;
begin
  if Camera='' then Exit;
  CreateValidFilename(Camera);
  With Main do
  begin
  if FileExists2(Progdir+Camera+'.dat') then
    DeleteFile(Progdir+Camera+'.dat');
  dat:= TInifile.Create(Progdir+Camera+'.dat');
  dat.WriteString('Device', 'Video', VideoDevice);
  dat.WriteInteger('Video', 'Width',VideoWidth );
  dat.WriteInteger('Video', 'Height',VideoHeight );
  dat.WriteInteger('Video','SnapSize', SnapSize);
  dat.WriteBool('Video','FitToPreviewWindow', FitToPreviewWindow.Checked);
  dat.WriteBool('Video','UseFrameRate', VidCap.UseFrameRate);
  dat.WriteInteger('Video', 'FotoSize', FotoSize.ItemIndex );
  dat.WriteFloat('Video','FrameRate',Vidcap.FrameRate);
  dat.WriteBool('Video','UseTimeLimit', VidCap.UseTimeLimit);
  dat.WriteInteger('Video','TimeLimit',Vidcap.TimeLimit);

  If NrOfMicroscopes=1 then DefaultMic:=0;
  dat.WriteInteger('Microscopes', 'DefaultMic', DefaultMic);
  dat.WriteInteger('Microscopes', 'DefaultObj', DefaultObj);
  end;
end;

procedure LoadCameraInfo(Camera:String);
var
  dat:TInifile;
begin
  if Camera='' then Exit;
  CreateValidFilename(Camera);
  With Main do
  begin
  dat:= TInifile.Create(Progdir+Camera+'.dat');
  VideoWidth:=dat.ReadInteger('Video', 'Width',640 );
  VideoHeight:=dat.ReadInteger('Video', 'Height',480 );
  SnapSize:=dat.ReadInteger('Video','SnapSize', 0 );
  FitToPreviewWindow.Checked:=dat.ReadBool('Video','FitToPreviewWindow', False );
  FotoSize.ItemIndex:=dat.ReadInteger('Video', 'FotoSize', 0 );
  VidCap.UseFrameRate:=dat.ReadBool('Video','UseFrameRate', False);
  Vidcap.FrameRate:=dat.ReadInteger('Video','FrameRate',25);
  VidCap.UseTimeLimit:=dat.ReadBool('Video','UseTimeLimit', True );
  Vidcap.TimeLimit:=dat.ReadInteger('Video','TimeLimit',30);
  
  DefaultMic:=dat.ReadInteger('Microscopes', 'DefaultMic',0);
  DefaultObj:=dat.ReadInteger('Microscopes', 'DefaultObj',0);
  SelectedMic:=DefaultMic;
  SelectedObj:=DefaultObj;
  end;
end;


procedure SaveBmpToFile(Bmp: TBitmap;FileName:String);
var
 Ext : String;
 jpeg: TJpegImage;
 PNG : TPngObject;
begin
  Ext:=lowercase(ExtractFileExt(Filename));
     if Ext='.bmp'then
        begin
            Bmp.SaveToFile(FileName);
            if Main.BMPcomment then
               Main.CommentList.SaveToFile(ExtractFilePath(Filename)+ExtractFileName(Filename)+'.cmt');
        end;
     if Ext='.jpg' then
        begin
           jpeg := TJPEGImage.Create;
           try
           jpeg.CompressionQuality := 100; //Default Value
           jpeg.Assign(Bmp);
           jpeg.SaveToFile(FileName);
           Main.JpgComment.FileName:=FileName;
           Main.jpgComment.Comment:=Main.CommentList.Text;
           Main.jpgComment.WriteComment;
           finally
           jpeg.Free
           end;
        end;
     if Ext='.png' then
        begin
           png := TPngObject.Create;
           try
           png.Assign(Bmp);
           png.AddText('MICAM',Main.CommentList.Text);
           png.SaveToFile(FileName);
           finally
           png.Free
           end;
        end;
end;

procedure LoadBmpFromFile(var bmp:TBitmap;FileName:String);
var
 Ext : String;
 jpeg: TJpegImage;
 PNG : TPngObject;
 PngList :TstringList;
 i : Integer;
begin
  if not FileExists(FileName) then Exit;
  Main.CommentList.Clear;
  Main.CommentMemo.Lines.Clear;
  Main.UserMemo.Lines.Clear;
  Ext:=lowercase(ExtractFileExt(Filename));
     if Ext='.bmp'then
        begin
          bmp.LoadFromFile(Filename);
          if FileExists(ExtractFilePath(Filename)+ExtractFileName(Filename)+'.cmt')
            then Main.CommentList.LoadFromFile(ExtractFilePath(Filename)+ExtractFileName(Filename)+'.cmt');
        end;
     if Ext='.jpg' then
        begin
           jpeg := TJPEGImage.Create;
           try
           jpeg.CompressionQuality := 100; //Default Value
           jpeg.LoadFromFile(FileName);
           bmp.Assign(jpeg);
           // load comment from jpg
           Main.JpgComment.FileName:=Filename;
           Main.JpgComment.ReadComment;
           Main.CommentList.Text:=Main.JpgComment.Comment;
           finally
           jpeg.Free
           end;
        end;
     if Ext='.png' then
        begin
           png := TPngObject.Create;
           pngList:=Tstringlist.Create;
           pnglist.Clear;
         try
           png.LoadFromFile(FileName);
           bmp.Assign(png);
           for i := 0 to png.chunks.count - 1 do
             if png.chunks.item[i] is TChunktEXt then
                pngList.AddObject(TChunktEXt(png.Chunks.Item[i]).Keyword, png.Chunks.Item[i]);
           for i := 0 to pngList.count - 1 do
             if pngList.Strings[i]='MICAM'
                then Main.CommentList.text:=TChunktEXt(pngList.Objects[i]).Text;
         finally
           pngList.Free;
           png.Free
         end;
        end;
end;

function FindInLines(Str:String;Lines:TStrings):String;
var
  i,j : Integer;
  S: ShortString;
begin
 Result:='';
 Str:=Lowercase(Str);
 For i:=0 to Lines.Count-1 do
   if Pos(Str,LowerCase(Lines[i]))<>0 then
      begin
        S:=LowerCase(Lines[i]);
        j:=Pos(Str,S)+Length(Str)-1;
        repeat
          inc(j);

        until ((S[j]<>' ') and (S[j]<>'='));
      Result :=Copy(Lines[i],j,Length(S));
      end;
end;

function ShowWarning(Msg:String): Boolean;
var
  AMsgDialog: TForm; 
  ACheckBox: TCheckBox;
begin
  Result:=False;
  AMsgDialog := CreateMessageDialog(Msg, mtWarning,[mbOK]);
  ACheckBox := TCheckBox.Create(AMsgDialog);

  with AMsgDialog do 
    try
      Caption := 'MICAM Warning' ;
      Height := 150;

      with ACheckBox do 
      begin 
        Parent := AMsgDialog; 
        Caption := 'Don''t show me again';
        Top := Parent.Height-55;
        Left := 8;
        Width := 120;
      end; 

      if (ShowModal = ID_OK)
          then if ACheckBox.Checked then Result:=True
          else if ACheckBox.Checked then Result:=False;
    finally
      ACheckBox.Free; 
      Free; 
    end; 
end;

end.
 