unit Preferences;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, ComCtrls, Dialogs, StrUtils;

type
  TPrefDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    Video: TPageControl;
    TabSheet1: TTabSheet;
    GroupBox3: TGroupBox;
    Microscope: TGroupBox;
    Label12: TLabel;
    TabSheet2: TTabSheet;
    ObjectivesBox: TListBox;
    NameBox: TComboBox;
    Label18: TLabel;
    Default: TCheckBox;
    AddMic: TButton;
    DelMic: TButton;
    EditMic: TButton;
    GroupBox4: TGroupBox;
    AddObj: TButton;
    DelObj: TButton;
    TabSheet3: TTabSheet;
    GroupBox6: TGroupBox;
    TabSheet4: TTabSheet;
    CaptureDoubleSized: TCheckBox;
    Label10: TLabel;
    ScaleLineWidthEdit: TEdit;
    ScalePenWidth: TUpDown;
    Label11: TLabel;
    ScaleLineColorBox: TColorBox;
    GroupBox7: TGroupBox;
    Label14: TLabel;
    Label15: TLabel;
    MeasureLineWidthEdit: TEdit;
    MeasurePenWidth: TUpDown;
    MeasureLineColorBox: TColorBox;
    SoundEnabledBox: TCheckBox;
    SnapshotWarning: TCheckBox;
    GroupBox8: TGroupBox;
    SelectFont: TButton;
    CopyrightFont: TLabel;
    FontDialog: TFontDialog;
    Label16: TLabel;
    Copyrightstr: TEdit;
    BMPComment: TCheckBox;
    GroupBox11: TGroupBox;
    UseTrigger: TCheckBox;
    COMEdit: TEdit;
    Label4: TLabel;
    COMGroup: TRadioGroup;
    Memo1: TMemo;
    DTRbox: TCheckBox;
    RTSbox: TCheckBox;
    DefaultCameraSettings: TCheckBox;
    GroupBox12: TGroupBox;
    Label5: TLabel;
    Label6: TLabel;
    Label21: TLabel;
    DeviceEdit: TEdit;
    MicroscopeEdit: TEdit;
    SaveDeviceSettings: TButton;
    ObjEdit: TEdit;
    Label23: TLabel;
    MeasureUnits: TComboBox;
    Warningbox: TGroupBox;
    ResizeWarning: TCheckBox;
    NoDevice: TCheckBox;
    GroupBox5: TGroupBox;
    Label13: TLabel;
    Label17: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label3: TLabel;
    CB0: TCheckBox;
    CB2: TCheckBox;
    CaptureNameEdit: TEdit;
    ResetFilenr: TCheckBox;
    DateTimePicker: TDateTimePicker;
    LocationEdit1: TEdit;
    LocationEdit2: TEdit;
    CB1: TCheckBox;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    UseTimeLimit: TCheckBox;
    TimeLimit: TComboBox;
    UseFrameRate: TCheckBox;
    FrameRate: TComboBox;
    GroupBox9: TGroupBox;
    Label7: TLabel;
    ImageType: TComboBox;
    GroupBox10: TGroupBox;
    SnapSize: TComboBox;
    GroupBox2: TGroupBox;
    Label8: TLabel;
    Label9: TLabel;
    FrameDirEdit: TEdit;
    SelectFrameDirBtn: TButton;
    SelectAVIDirBtn: TButton;
    AVIDirEdit: TEdit;
    AVIWarning: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure SelectFrameDirBtnClick(Sender: TObject);
    procedure AddMicClick(Sender: TObject);
    procedure DelMicClick(Sender: TObject);
    procedure EditMicClick(Sender: TObject);
    procedure SetObjectivesboxItems(Microscope: Integer);
    procedure NameBoxChange(Sender: TObject);
    procedure AddObjClick(Sender: TObject);
    procedure DelObjClick(Sender: TObject);
    procedure ScalePenWidthClick(Sender: TObject; Button: TUDBtnType);
    procedure CB0Click(Sender: TObject);
    procedure CB2Click(Sender: TObject);
    procedure MeasurePenWidthClick(Sender: TObject; Button: TUDBtnType);
    procedure ObjectivesBoxClick(Sender: TObject);
    procedure SelectFontClick(Sender: TObject);
    procedure CB1Click(Sender: TObject);
    procedure SaveDeviceSettingsClick(Sender: TObject);
    procedure DefaultClick(Sender: TObject);
    procedure SelectAVIDirBtnClick(Sender: TObject);
    procedure UseFrameRateClick(Sender: TObject);
    procedure UseTimeLimitClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure ApplyChanges;
  end;

var
  PrefDlg: TPrefDlg;

const
  SELDIRHELP = 0;


implementation

{$R *.DFM}
uses MICAMMainUnit, FileCtrl, Helpunit, SaveOptions, Serial, MICAMUtils;

var
  TmpMic : Array [0..15] of TMicroscope;

procedure TPrefDlg.FormCreate(Sender: TObject);
var
 i : Integer;
begin
  ///////////////////////////////////////////
  // Microscope Tab
  ///////////////////////////////////////////

  for i:= 0 to 15 do
     TmpMic[i]:=Main.Microscope[i];
  Namebox.Items.Clear;
  if Main.NrofMicroscopes <>0 then
     begin
     for i:=0 to Main.NrofMicroscopes-1 do
         begin
         Namebox.Items.Add(TmpMic[i].Name);
         end;
     Namebox.ItemIndex:=Main.DefaultMic;
     SetObjectivesBoxItems(Namebox.ItemIndex);
     end;

   Namebox.ItemIndex:=Main.SelectedMic;

   ObjectivesBox.ItemIndex:=Main.SelectedObj;

   ObjectivesBoxClick(nil);
   SnapshotWarning.Checked:=Main.SnapshotWarning;


  ///////////////////////////////////////////
  //   Image and Video Tab
  ///////////////////////////////////////////

   ImageType.ItemIndex:=Main.ImageKind;
   AVIDirEdit.Text:=Main.AVIDir;
   FrameDirEdit.Text:=Main.FrameDir;
   DateTimePicker.Date:=Main.SampleDate;
   LocationEdit1.Text:= Main.LocationText1;
   LocationEdit2.Text:= Main.LocationText2;
  
  CaptureNameEdit.Text:=Main.CaptureName;
  if Main.CaptureFileType=0 then
     begin
     CB0.Checked:=True;
     CaptureNameEdit.Enabled:=False;
     end;
  if Main.CaptureFileType=1 then
     begin
     CB1.Checked:=True;
     CaptureNameEdit.Enabled:=False;
     end;
  if Main.CaptureFileType=2 then
     begin
     CB2.Checked:=True;
     end;

  UseTimeLimit.Checked := Main.VidCap.UseTimeLimit;
  TimeLimit.Text:=IntToStr(Main.VidCap.TimeLimit);
  UseTimeLimitClick(nil);

  UseFrameRate.Checked := Main.VidCap.UseFrameRate;
  FrameRate.Text:=FloatToStr(Main.VidCap.Framerate);

 // FastPreview.Checked:=Main.VidCap.UseClock;

  UseFrameRateClick(nil);

  SnapSize.ItemIndex:=Main.SnapSize;

  ///////////////////////////////////////////
  //   Measuring Tab
  ///////////////////////////////////////////

  ScaleLineWidthEdit.Text:=IntToStr(Main.ScalePen.Width);
  ScalePenWidth.Position:=Main.ScalePen.Width;
  ScaleLineColorbox.Selected:=TColor(Main.ScalePen.Color);

  MeasureLineWidthEdit.Text:=IntToStr(Main.MeasurePen.Width);
  MeasurePenWidth.Position:=Main.MeasurePen.Width;
  MeasureLineColorbox.Selected:=TColor(Main.MeasurePen.Color);
  Measureunits.Text:=Main.Units;

  ///////////////////////////////////////////
  //   Miscellaneous Tab
  ///////////////////////////////////////////

  CaptureDoubleSized.Checked:=Main.CaptureDoubleSized;
  SoundEnabledBox.Checked:=Main.SoundEnabled;
  Copyrightstr.Text:=Main.CopyrightString;
  CopyRightFont.Font:=Main.CopyRightFont;
  BMPComment.Checked:=Main.BMPcomment;
  DefaultCameraSettings.Checked:=Main.UseDeviceDefaultSettings;
  NoDevice.Checked:=Main.NoDevice;

  ResizeWarning.Checked:=Main.WarningOnResize;
  AVIWarning.Checked:=Main.WarningOnAVI;

  DeviceEdit.Text:=Main.VideoDevice;
  MicroscopeEdit.Text:=Main.Microscope[Namebox.ItemIndex].Name;
  ObjEdit.Text:=FloatToStr(Main.Microscope[Namebox.ItemIndex].Objective[ObjectivesBox.ItemIndex].Obj)+' X';

  CloseCom;
  UseTrigger.Checked:=Main.UseHWTimer;
  ComEdit.Text:=IntToStr(Main.PoortNr);
  ComGroup.ItemIndex:=Main.PoortLine;
  RTSbox.Checked:= (Main.PoortHigh and 1)<> 0;
  DTRbox.Checked:= (Main.PoortHigh and 2)<> 0;

end;

procedure TPrefDlg.OKBtnClick(Sender: TObject);
begin
  modalresult:= mrNone;
  ApplyChanges;
  modalresult:= mrOk;
end;

procedure TPrefDlg.ApplyChanges;
var
  i :integer;
  Str : String;
begin


 ///////////////////////////////////////////
  // Microscope Tab
  ///////////////////////////////////////////
 if Default.Checked=True then
    begin
    Main.DefaultMic:=Namebox.ItemIndex;
    Main.DefaultObj:=ObjectivesBox.ItemIndex;
    end;
 Main.SelectedMic:=Namebox.ItemIndex;
 Main.SelectedObj:=ObjectivesBox.ItemIndex;
 Main.NrofMicroscopes:=Namebox.Items.Count;
 Main.Objective.Font.Color:=clBlack;

 for i:= 0 to Main.NrofMicroscopes-1 do
    begin
    TMpMic[i].Name:=Namebox.Items[i];
    Main.Microscope[i]:=TmpMic[i];
    end;
 Main.SnapshotWarning:=SnapshotWarning.Checked;
 

  ///////////////////////////////////////////
  //   Image and Video Tab
  ///////////////////////////////////////////

  Main.AVIDir:=AVIDirEdit.Text;
  Main.FrameDir:=FrameDirEdit.Text;
  Main.SaveDialog.InitialDir:=FrameDirEdit.Text;
  Str:=Main.CreateFilename(0);
  Main.SaveDialog.FileName:=Str;

  Main.ImageKind:=ImageType.ItemIndex;
  Main.SampleDate:=DateTimePicker.Date;
  Main.LocationText1:=LocationEdit1.Text;
  Main.LocationText2:=LocationEdit2.Text;

  if CB0.Checked=True then Main.CaptureFileType:=0;
  if CB1.Checked=True then Main.CaptureFileType:=1;
  if CB2.Checked=True then
     begin
     Main.CaptureFileType:=2;
     Main.CaptureName:=CaptureNameEdit.Text;
     if ResetFileNr.Checked=True then Main.CaptureNr:=0;
     end;

  SaveWithOptions.IncludeMag.Checked:= not(CB1.Checked);

  Main.VidCap.UseTimeLimit := UseTimeLimit.Checked;
  Main.VidCap.TimeLimit := StrToInt(TimeLimit.Text);

  Main.VidCap.UseFrameRate := UseFrameRate.Checked;
  Main.VidCap.Framerate:= StrToInt(FrameRate.Text);

 // Main.VidCap.StopPreview;
 // Main.VidCap.UseClock:= not FastPreview.Checked;
 // Main.VidCap.StartPreview;
  
  Main.SnapSize:=SnapSize.ItemIndex;

  ///////////////////////////////////////////
  //   Measuring Tab
  ///////////////////////////////////////////

  Main.MeasurePen.Width:=MeasurePenWidth.Position;
  Main.MeasurePen.Color:=MeasureLineColorbox.Selected;
  Main.Units:=Measureunits.Text;
  Main.InsertScale1.Caption:='Insert Scale 10 '+Main.Units;
  Main.InsertScale2.Caption:='Insert Scale 20 '+Main.Units;
  Main.InsertScale3.Caption:='Insert Scale 50 '+Main.Units;
  Main.InsertScale4.Caption:='Insert Scale 100 '+Main.Units;
  Main.CalLabel.Caption:=Main.Units+' / pixel';

  Main.ScalePen.Width:=ScalePenWidth.Position;
  Main.ScalePen.Color:=ScaleLineColorbox.Selected;

  Main.CopyrightString:=Copyrightstr.Text;
  Main.CopyRightFont.Assign(CopyrightFont.Font);

  ///////////////////////////////////////////
  //   Miscellaneous Tab
  ///////////////////////////////////////////

  Main.CaptureDoubleSized:=CaptureDoubleSized.Checked;
  Main.SoundEnabled:=SoundenabledBox.Checked;
  Main.BMPcomment:=BMPComment.Checked;
  Main.NoDevice:=NoDevice.Checked;
  Main.UseDeviceDefaultSettings:=DefaultCameraSettings.Checked;

  Main.WarningOnResize:=ResizeWarning.Checked;
  Main.WarningOnAVI:=AVIWarning.Checked;

  if DefaultCameraSettings.Checked=True then
     begin
     Str:=Main.VideoDevice;
     CreateValidFilename(Str);
     if not FileExists2(Main.Progdir+Str+'.dat') then
        if MessageDlg('No default settings found for device:'+#13#10+Main.VideoDevice+#13#10+'Save current Device settings as default?',mtWarning,mbYesNoCancel,0)=mrYes
           then SaveCameraInfo(Main.VideoDevice)
           else DefaultCameraSettings.Checked:= False;
     end;
  
  Main.UseHWTimer:=UseTrigger.Checked;
  if UseTrigger.Checked=False then CloseCom
  else
  begin
    Main.PoortNr:=StrToInt(ComEdit.Text);
    Main.PoortLine:=ComGroup.ItemIndex;
    if OpenCom(Main.Poortnr)=False then
       ShowMessage('Cannot open port COM'+IntToStr(Main.Poortnr));
    Case Comgroup.ItemIndex of
      0 : Main.PoortActive:= not CTS;
      1 : Main.PoortActive:= not DSR;
      2 : Main.PoortActive:= not DCD;
      3 : Main.PoortActive:= not RI;
      end;
  Main.PoortHigh:=0;
  if RTSbox.Checked then Main.PoortHigh:=1;
  if DTRbox.Checked then Main.PoortHigh:=Main.PoortHigh+ 2;
  end;

end;

  ///////////////////////////////////////////
  //
  // Functions
  ///////////////////////////////////////////

  ///////////////////////////////////////////
  // Microscope Tab
  ///////////////////////////////////////////

procedure TPrefDlg.SetObjectivesboxItems(Microscope: Integer);
var
  k: Integer;
begin
  ObjectivesBox.Items.Clear;
  k:=0;
  repeat
     if TmpMic[Microscope].Objective[k].Obj<>0 then
        ObjectivesBox.Items.Add(TmpMic[Microscope].Objective[k].ObjStr);
     inc(k);
  until k>31;
end;

procedure TPrefDlg.AddMicClick(Sender: TObject);

begin
  Namebox.Items.Add(InputBox('Add microscope name','Enter a new microscope name', 'NewMicroscope'));
  Namebox.ItemIndex:=Namebox.Items.Count-1;
  ObjectivesBox.Items.Clear;
  AddObjClick(nil);
end;

procedure TPrefDlg.DelMicClick(Sender: TObject);
var
  str : String;
  pos: Integer;
begin
  if Main.DefaultMic=Namebox.ItemIndex then
     begin
     ShowMessage('You cannot delete the default microscope'#13#10'Set another microscope as default first');
     Exit;
     end;
  pos:=Namebox.ItemIndex;
  str:='Delete microscope: '#13#10+Namebox.Text+' ?';
  if MessageDlg(str, mtConfirmation, [mbYes, mbCancel], 0) = mrYes then
      Namebox.DeleteSelected;
  Namebox.ItemIndex:=pos;
  while pos<15 do
     begin
     TmpMic[pos]:=TmpMic[pos+1];
     inc(pos);
     end;
  NameBox.ItemIndex:=0;
  SetObjectivesBoxItems(0);
end;

procedure TPrefDlg.EditMicClick(Sender: TObject);
var
  pos: Integer;
begin
  pos:=Namebox.ItemIndex;
  Namebox.Items[Namebox.ItemIndex]:=InputBox('Edit microscope name',
          'Change the microscope name', Namebox.Items[Namebox.ItemIndex]);
  Namebox.ItemIndex:=pos;
  SetObjectivesBoxItems(Namebox.ItemIndex);
end;

procedure TPrefDlg.NameBoxChange(Sender: TObject);
begin
  SetObjectivesBoxItems(Namebox.ItemIndex);
  if ObjectivesBox.ItemIndex<0 then  ObjectivesBox.ItemIndex:=0;
  Default.Caption:='Set '+NameBox.Items[NameBox.ItemIndex]
                  +' with '+ObjectivesBox.Items[ObjectivesBox.ItemIndex]
                  + ' as default';
  MicroscopeEdit.Text:=Namebox.Text;
  ObjEdit.Text:=Objectivesbox.Items[ObjectivesBox.ItemIndex];
end;

procedure TPrefDlg.AddObjClick(Sender: TObject);
var
  i : Integer;
  ObjValue: Real;
  NewObjective, Str : String;
begin
  if Objectivesbox.Count>31 then
     begin
     ShowMessage('only 32 objectives allowed');
     Exit;
     end;
  if InputQuery('Add Objective','Enter an objective magnification as a number. e.g. 10 or '+FloatToStr(4.5)+#10#13'Leave a space after the magnification and you can add comment', NewObjective)=False then exit;
  try
   NewObjective:=NewObjective+' ';
   Str:=LeftStr(NewObjective,Pos(' ',NewObjective));
   ObjValue:=StrToFloat(Str);
   except
   On EConvertError do
       begin
         ShowMessage(NewObjective+' is not a legal value (must be a number)');
         Exit;
       end;
   end;
  Objectivesbox.Items.Add(NewObjective);
  i:=Objectivesbox.Count-1;
  TmpMic[Namebox.ItemIndex].NrofObjectives:=Objectivesbox.Count;
  TmpMic[Namebox.ItemIndex].Objective[i].Obj:=ObjValue;
  TmpMic[Namebox.ItemIndex].Objective[i].ObjStr:=NewObjective;
  TmpMic[Namebox.ItemIndex].Objective[i].Cal:=0;
end;

procedure TPrefDlg.DelObjClick(Sender: TObject);
var
  str : String;
  pos: Integer;
begin
  pos:=Objectivesbox.ItemIndex;
  if (Pos<0) or (Pos>Objectivesbox.Count-1) then begin ShowMessage('Select an objective first');Exit; end;
  str:='Delete Objective: '#13#10+ObjectivesBox.Items[ObjectivesBox.Itemindex]+' ?';
  if MessageDlg(str, mtConfirmation, [mbYes, mbCancel], 0) = mrYes then
    begin
    while pos<Objectivesbox.Count-1 do
      begin
      TmpMic[Namebox.ItemIndex].Objective[pos]:=TmpMic[Namebox.ItemIndex].Objective[pos+1];
      inc(pos);
      end;
    TmpMic[Namebox.ItemIndex].Objective[pos].Obj:=0;
    TmpMic[Namebox.ItemIndex].Objective[pos].Cal:=0;
    SetObjectivesBoxItems(Namebox.ItemIndex);
    end;
end;

procedure TPrefDlg.ObjectivesBoxClick(Sender: TObject);
begin
  if ObjectivesBox.ItemIndex<0 then  begin ObjectivesBox.ItemIndex:=0; Exit; end;
  Default.Caption:='Set '+NameBox.Items[NameBox.ItemIndex]
                    +' with '+ObjectivesBox.Items[ObjectivesBox.ItemIndex]
                    + ' as default';
  MicroscopeEdit.Text:=Namebox.Text;
  ObjEdit.Text:=Objectivesbox.Items[ObjectivesBox.ItemIndex];
end;

  ///////////////////////////////////////////
  //   Image and Video Tab
  ///////////////////////////////////////////

procedure TPrefDlg.SelectAVIDirBtnClick(Sender: TObject);
var
  Dir : String;
begin
    Dir:=Main.AVIDir;
    if not (DirectoryExists(Dir))
       then Dir:=Main.ProgDir+'\Capture';
    if SelectDirectory(Dir, [sdAllowCreate, sdPerformCreate, sdPrompt],SELDIRHELP) then
    AVIDirEdit.Text := Dir;
end;
  
procedure TPrefDlg.SelectFrameDirBtnClick(Sender: TObject);
var
  Dir, Tst : String;
begin
    Dir:=Main.FrameDir;
    if not (DirectoryExists(Dir))
       then Dir:=Main.ProgDir+'Capture';
    if SelectDirectory(Dir, [sdAllowCreate, sdPerformCreate, sdPrompt],SELDIRHELP) then
    FrameDirEdit.Text := Dir;
    Tst:='#';
    Dir:=ExtractFilename(Dir);
    if Pos(Tst,Dir)<>0 then
      begin
        LocationEdit1.Text:=Copy(Dir,1,Pos(Tst,Dir)-1);
        LocationEdit2.Text:=Copy(Dir,Pos(Tst,Dir)+1,Length(Dir));
      end
         else
      begin
        LocationEdit1.Text:=Dir;
        LocationEdit2.Text:='';
      end;
end;

procedure TPrefDlg.CB0Click(Sender: TObject);
begin
  if CB0.Checked=True then
     begin
     CB1.Checked:=False;
     CB2.Checked:=False;
     CaptureNameEdit.Enabled:=False;
     ResetFileNr.Enabled:=False;
     end;
end;

procedure TPrefDlg.CB1Click(Sender: TObject);
begin
  if CB1.Checked=True then
     begin
     CB0.Checked:=False;
     CB2.Checked:=False;
     CaptureNameEdit.Enabled:=False;
     ResetFileNr.Enabled:=False;
     end;
end;

procedure TPrefDlg.CB2Click(Sender: TObject);
begin
   if CB2.Checked=True then
     begin
     CB0.Checked:=False;
     CB1.Checked:=False;
     CaptureNameEdit.Enabled:=True;
     ResetFileNr.Enabled:=True;
     end;
end;


  ///////////////////////////////////////////
  //   Measuring Tab
  ///////////////////////////////////////////

procedure TPrefDlg.ScalePenWidthClick(Sender: TObject; Button: TUDBtnType);
begin
  ScaleLineWidthEdit.Text:=IntToStr(ScalePenWidth.Position);
end;

procedure TPrefDlg.MeasurePenWidthClick(Sender: TObject;
  Button: TUDBtnType);
begin
  MeasureLineWidthEdit.Text:=IntToStr(MeasurePenWidth.Position);
end;

procedure TPrefDlg.SelectFontClick(Sender: TObject);
begin
 if FontDialog.Execute then
  begin
  CopyrightFont.Font:= FontDialog.Font;

  end
end;

procedure TPrefDlg.SaveDeviceSettingsClick(Sender: TObject);
begin
 if MessageDlg('Changes in Preferences will be saved also'+#13#10+'Do you want to continue?',mtWarning,mbYesNoCancel,0)=mrYes
    then begin
    ApplyChanges;
    Main.DefaultMic:=Namebox.ItemIndex;
    Main.DefaultObj:=ObjectivesBox.ItemIndex;
    SaveCameraInfo(DeviceEdit.Text);
    end;
end;

procedure TPrefDlg.DefaultClick(Sender: TObject);
begin
  MicroscopeEdit.Text:=Main.Microscope[Namebox.ItemIndex].Name;
  ObjEdit.Text:=FloatToStr(Main.Microscope[Namebox.ItemIndex].Objective[ObjectivesBox.ItemIndex].Obj)+' X';
end;

procedure TPrefDlg.UseTimeLimitClick(Sender: TObject);
begin
  Timelimit.Enabled:=UseTimeLimit.Checked;
  if TimeLimit.Enabled then TimeLimit.Color:=clWindow else TimeLimit.Color:=clBtnFace;
end;

procedure TPrefDlg.UseFrameRateClick(Sender: TObject);
begin
  FrameRate.Enabled:=UseFrameRate.Checked;
  if FrameRate.Enabled then Framerate.Color:=clWindow else Framerate.Color:=clBtnFace;
end;

end.
