unit SaveOptions;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, FileCtrl, ComCtrls;

type
  TSaveWithOptions = class(TForm)
    SaveWithName: TButton;
    Cancel: TButton;
    Label3: TLabel;
    DatabaseBox: TComboBox;
    Names: TListBox;
    Label5: TLabel;
    LocalNames: TCheckBox;
    UseSearchName: TButton;
    Rename: TButton;
    GroupBox1: TGroupBox;
    IncludeDate: TCheckBox;
    IncludeMag: TCheckBox;
    DatePicker: TDateTimePicker;
    Label8: TLabel;
    Label7: TLabel;
    ImageType: TComboBox;
    Label2: TLabel;
    SugestedFileName: TEdit;
    Directory: TEdit;
    Label6: TLabel;
    SelectDir: TButton;
    GroupBox2: TGroupBox;
    CommentMemo: TMemo;
    Label9: TLabel;
    LocationEdit1: TEdit;
    LocationEdit2: TEdit;
    UserMemo: TMemo;
    Label1: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    RecentBox: TListBox;
    Label12: TLabel;
    SearchEdit: TEdit;
    ClearSearchText: TButton;
    SortRecentNames: TCheckBox;
    ClearRecentNames: TButton;
    RecentBoxSorted: TListBox;
    procedure CancelClick(Sender: TObject);
    procedure SaveWithNameClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DBSearch(const PathName, FileName : string);
    procedure LoadDB(naam: String);
    procedure SearchEditChange(Sender: TObject);
    procedure SearchClick(Sender: TObject);
    procedure DatabaseBoxChange(Sender: TObject);
    procedure IncludeDateClick(Sender: TObject);
    procedure IncludeMagClick(Sender: TObject);
    procedure IncludeTimeClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure UseSearchNameClick(Sender: TObject);
    procedure SelectDirClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ImageTypeChange(Sender: TObject);
    procedure SetDateClick(Sender: TObject);
    procedure FindUniqueName(Str:String);
    procedure RenameClick(Sender: TObject);
    procedure InsertComment(Sender: TObject);
    procedure LocationEditChange(Sender: TObject);
    procedure RecentBoxClick(Sender: TObject);
    procedure ClearSearchTextClick(Sender: TObject);
    procedure NamesMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure NamesClick;
    procedure SortRecentNamesClick(Sender: TObject);
    procedure ClearRecentNamesClick(Sender: TObject);
    procedure NamesKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure NamesEnter(Sender: TObject);
    procedure RecentBoxSortedEnter(Sender: TObject);
    procedure RecentBoxEnter(Sender: TObject);
    procedure RecentBoxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure RecentBoxSortedKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
  public
    { Public declarations }
    Snap : boolean;
    procedure AddToRecentNames(Str: String);
  end;

var
  SaveWithOptions: TSaveWithOptions;

implementation

{$R *.dfm}
uses MICAMMainUnit, MICAMUtils;

var
 NedLines: TStrings;
 LatLines: TStrings;
 DateNumber : Integer;

procedure TSaveWithOptions.CancelClick(Sender: TObject);
begin
 ModalResult:= mrCancel;
end;

procedure TSaveWithOptions.SaveWithNameClick(Sender: TObject);
begin
 if SugestedFileName.Text=''
    then ShowMessage('Invalid filename')
    else ModalResult:=mrOK;
end;

procedure TSaveWithOptions.FormCreate(Sender: TObject);
begin
  NedLines:=TStringlist.Create;
  LatLines:=TStringlist.Create;
  DatabaseBox.Clear;
  DBSearch(Main.ProgDir,'Databases\*.csv');
  DatabaseBox.ItemIndex:=0;
  LoadDB(DatabaseBox.Items.Strings[DatabaseBox.ItemIndex]);
  SearchEdit.Text:='';
  SearchClick(nil);
end;

procedure TSaveWithOptions.DBSearch(const PathName, FileName : string);
var Rec  : TSearchRec;
    Path : string;
begin
Path := IncludeTrailingPathDelimiter(PathName);
if FindFirst(Path + FileName, faAnyFile - faDirectory, Rec) = 0 then
 try
   repeat
     DatabaseBox.Items.Add(Rec.Name);
   until FindNext(Rec) <> 0;
 finally
   FindClose(Rec);
 end;
end;

procedure TSaveWithOptions.LoadDB(naam: String);
var
 i, k, Lengte : Integer;
 S : String;
begin
  if naam='' then exit;
  NedLines.Clear;
  LatLines.Clear;
  NedLines.LoadFromFile(Main.ProgDir+'\Databases\'+naam);
  for i:=0 to NedLines.Count-1 do
     begin
     S:=NedLines[i];
     Lengte:=Length(S);
     k:=Pos(';',S);
     LatLines.Add(Copy(S,k+1,Lengte));
     Delete(S,k,Lengte);
     NedLines[i]:=S;
     end;
end;

procedure TSaveWithOptions.SearchClick(Sender: TObject);
var
 i,k :Integer;
begin
 Names.Clear;

 if SearchEdit.Text=''
    then begin
         if LocalNames.Checked
         then for i:=0 to NedLines.Count-1 do
                  if Nedlines[i]<>'' then Names.Items.Add(Nedlines[i])
                                     else
         else for i:=0 to LatLines.Count-1 do
                  Names.Items.Add(Latlines[i]);
         Names.Sorted:=True;
         exit;
         end;
  k:=0;

  if LocalNames.Checked
  then
      begin
      for i:=0 to NedLines.Count-1 do
        begin
        if k>100 then continue;
        if Pos(lowercase(SearchEdit.Text),lowercase(nedLines[i]))<>0 then
           Names.Items.Add(Nedlines[i]);
        end;
      end
   else
      begin
      for i:=0 to LatLines.Count-1 do
        begin
        if k>100 then continue;
        if Pos(lowercase(SearchEdit.Text),lowercase(LatLines[i]))<>0 then
           Names.Items.Add(Latlines[i]);
        end;
      end;
   Names.Sorted:=True;
end;



procedure TSaveWithOptions.SearchEditChange(Sender: TObject);
begin
  SearchClick(nil);
end;

procedure TSaveWithOptions.DatabaseBoxChange(Sender: TObject);
begin
  LoadDB(DatabaseBox.Items.Strings[DatabaseBox.ItemIndex]);
  SearchEdit.Text:='';
  SearchClick(nil);
end;

procedure TSaveWithOptions.UseSearchNameClick(Sender: TObject);
begin
  Names.ItemIndex:=-1;
  NamesClick;
end;

procedure TSaveWithOptions.FindUniqueName(Str:String);
var
 Testname : String;
begin
  DateNumber:=0;
  repeat
  TestName:=Directory.Text+'\'+Str;
  inc(DateNumber);

  if IncludeDate.Checked then TestName:=TestName+'_'+DateToStr(Main.SampleDate);
  TestName:=TestName+'_'+FormatFloat('0000',DateNumber);
  if IncludeMag.Checked then TestName:=TestName+'*'; //+ FloatToStr(Main.Microscope[Main.SelectedMic].Objective[Main.SelectedObj].Obj)+'X';
  TestName:=TestName+'.'+ImageType.Text;
  until (not FileExists2(TestName)) or (DateNumber>9999);
end;


procedure TSaveWithOptions.IncludeDateClick(Sender: TObject);
begin
 NamesClick;
end;

procedure TSaveWithOptions.IncludeMagClick(Sender: TObject);
begin
  NamesClick;
end;

procedure TSaveWithOptions.IncludeTimeClick(Sender: TObject);
begin
  NamesClick;
end;

procedure TSaveWithOptions.FormDestroy(Sender: TObject);
begin
  NedLines.Free;
  LatLines.Free;
end;

procedure TSaveWithOptions.ClearSearchTextClick(Sender: TObject);
begin
  SearchEdit.Text:='';
  SearchClick(nil);
end;

procedure TSaveWithOptions.SelectDirClick(Sender: TObject);
var
  Dir : String;
begin
    Dir:=Main.SaveDialog.InitialDir;
    if SelectDirectory(Dir, [sdAllowCreate, sdPerformCreate, sdPrompt],0) then
    Directory.Text := Dir;
    Main.SaveDialog.InitialDir:=Dir;
    if ImgSourceIsFile then
       if ExtractFileDir(Caption) = Directory.Text
          then Rename.Visible:=True
          else Rename.Visible:=False;
end;

procedure TSaveWithOptions.FormShow(Sender: TObject);
begin
  LocationEdit1.Text:= Main.LocationText1;
  LocationEdit2.Text:= Main.LocationText2;
  SugestedFileName.Text:='';
  Directory.Text:=Main.SaveDialog.InitialDir;
  ImageType.ItemIndex:=Main.Imagekind;
  Rename.Visible:=False;
 Rename.Tag:=0;
 if ImgSourceIsFile then
   if ExtractFileDir(Caption) = Directory.Text
      then Rename.Visible:=True
      else Rename.Visible:=False;
 DatePicker.Date:=Main.SampleDate;
 UserMemo.Text:=Main.UserMemo.Text;
 InsertComment(nil);
 SearchEdit.SetFocus;
end;

procedure TSaveWithOptions.ImageTypeChange(Sender: TObject);
begin
 NamesClick;
end;

procedure TSaveWithOptions.SetDateClick(Sender: TObject);
begin
 Main.SampleDate:=DatePicker.Date;
 NamesClick;
end;



procedure TSaveWithOptions.RenameClick(Sender: TObject);
begin
 Rename.Tag:=1;
 if SugestedFileName.Text=''
    then begin ShowMessage('Invalid filename'); Rename.Tag:=0;end
    else ModalResult:=mrOK;
end;

procedure TSaveWithOptions.InsertComment(Sender: TObject);
begin
  CommentMemo.Clear;
  CommentMemo.Lines.Add('');
  CommentMemo.Lines.Add('');
  CommentMemo.Lines.Add('');
  CommentMemo.Lines[0]:=(LocationEdit1.Text);
  CommentMemo.Lines[1]:=(LocationEdit2.Text);
  CommentMemo.Lines[2]:=('SampleDate = '+DateToStr(Main.SampleDate));
  CommentMemo.Lines[3]:=(Main.Microscope[Main.SelectedMic].Name+';  Obj: '
                        + FloatToStr(Main.Microscope[Main.SelectedMic].Objective[Main.SelectedObj].Obj)
                        + ' X;  um/pixel: '+FloatToStrF(Main.CalValue,ffFixed,4,6));
end;

procedure TSaveWithOptions.LocationEditChange(Sender: TObject);
begin
  InsertComment(nil);
end;



procedure TSaveWithOptions.AddToRecentNames(Str: String);
var
  i : Integer;
begin
 for i:=0 to Recentbox.Count-1 do
   if Str=Recentbox.Items[i] then Exit;
 Recentbox.Items.Insert(0,Str);
 RecentboxSorted.Items.Add(Str);
end;

procedure TSaveWithOptions.RecentBoxClick(Sender: TObject);
var
  Str : String;
begin
    if Recentbox.ItemIndex>=0 then
       if SortRecentNames.Checked=True then Str:=RecentboxSorted.Items[RecentboxSorted.ItemIndex]
                               else Str:=Recentbox.Items[Recentbox.ItemIndex];
    SearchEdit.Text:=Str;
    DateSeparator := '-';
    ShortDateFormat := 'yyy/mm/dd';
    FindUniqueName(Str);

    if IncludeDate.Checked then Str:=Str+'_'+DateToStr(Main.SampleDate);
    Str:=Str+'_'+FormatFloat('0000',DateNumber);
    if IncludeMag.Checked then Str:=Str+'_'+ FloatToStr(Main.Microscope[Main.SelectedMic].Objective[Main.SelectedObj].Obj)+'X';
    SugestedFileName.Text:= Str;
    InsertComment(nil);
end;

procedure TSaveWithOptions.NamesMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  NamesClick;
end;

procedure TSaveWithOptions.NamesClick;
var
  Str : String;
begin
    if Names.ItemIndex>=0
       then Str:=Names.Items[Names.ItemIndex]
       else Str:=SearchEdit.Text;
    AddToRecentNames(Str);
    DateSeparator := '-';
    ShortDateFormat := 'yyy/mm/dd';
    FindUniqueName(Str);

    if IncludeDate.Checked then Str:=Str+'_'+DateToStr(Main.SampleDate);
    Str:=Str+'_'+FormatFloat('0000',DateNumber);
    if IncludeMag.Checked then Str:=Str+'_'+ FloatToStr(Main.Microscope[Main.SelectedMic].Objective[Main.SelectedObj].Obj)+'X';
    SugestedFileName.Text:= Str;
    InsertComment(nil);
end;

procedure TSaveWithOptions.NamesKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=VK_RETURN then
     begin
     NamesClick;
     SaveWithName.SetFocus;
     end;
  if Key=VK_TAB then
     if SortRecentNames.Checked=True
        then  RecentboxSorted.SetFocus
        else  Recentbox.SetFocus;
end;

procedure TSaveWithOptions.SortRecentNamesClick(Sender: TObject);
begin
 if SortRecentNames.Checked=True
    then begin Recentbox.Visible:=False; RecentboxSorted.Visible:=True;end
    else begin Recentbox.Visible:=True; RecentboxSorted.Visible:=False;end
end;

procedure TSaveWithOptions.ClearRecentNamesClick(Sender: TObject);
begin
 Recentbox.Clear;
 RecentboxSorted.Clear;
end;

procedure TSaveWithOptions.NamesEnter(Sender: TObject);
begin
 Names.ItemIndex:=0;
end;

procedure TSaveWithOptions.RecentBoxSortedEnter(Sender: TObject);
begin
 RecentboxSorted.ItemIndex:=0;
end;

procedure TSaveWithOptions.RecentBoxEnter(Sender: TObject);
begin
  Recentbox.ItemIndex:=0;
end;

procedure TSaveWithOptions.RecentBoxKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=VK_RETURN then
     begin
     RecentboxClick(nil);
     SaveWithName.SetFocus;
     end;
end;

procedure TSaveWithOptions.RecentBoxSortedKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key=VK_RETURN then
     begin
     RecentboxClick(nil);
     SaveWithName.SetFocus;
     end;
end;

end.
